/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web._base.url");

dojo.require("com.ibm.team.process.web.client.projectarea");

(function(){
	
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var TypeInfo = {
	LOADING: { iconUrl: "ui/view/common/images/spinner.gif" },
	FOLDER: { iconUrl: "ui/images/cfldr_obj.gif" },
	LIST: {
		action: "showQueryResult",
		iconUrl: "ui/images/query_obj.gif",
		remoteUrl: "ui/images/remot_query.gif"
	},
	RECORD: {
		action: "showRecord",
		iconUrl: "ui/images/record.gif",
		remoteUrl: "ui/images/remot_record.gif"
	},
	CHART: {
		action: "showQueryResult",
		iconUrl: "ui/images/chart.gif",
		remoteUrl: "ui/images/remot_charts.gif"
	},
	REPORT: {
		action: "showQueryResult",
		iconUrl: "ui/images/report.gif",
		remoteUrl: "ui/images/report.gif"
	}
}

dojo.mixin(com.ibm.rational.cm.web,
{
	ACTION_PREFIX: "com.ibm.rational.cm.",

	// com.ibm.rational.cm.rest/plugin.xml defines it.	
	CM_SERVICE_CONTEXT_ROOT: "/cqrest",
	
	getURLforLink: function(type, id){
//		return this.getActionHref(type, id, "");
		return this.getResourceHref(type, id);		
	},

	getActionHref: function(/*String*/ type, /*String*/ id, /*String*/ label, /*Boolean?*/ isAbsolute){
		// summary:
		//		returns action URL for a given resource
		// type: String
		//		takes value of LIST|RECORD|CHART|REPORT
		// id: String
		//		stableLocation of the resource
		// label: String
		//		display name for the resource
		//		TODO : currently 'label' is included in the URL, but we shouldn't include it
		// isAbsolute: Boolean [optional]
		//		if true, returns absolute path. Otherwise, relative path will be returned
		var info = TypeInfo[type];
		if(info){
			var baseUrl;
			if(isAbsolute){
				var winUrl = new dojo._Url(window.location.href);
				var urlRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" );

				// Escalation RATLC01312339 : Cannot view detailed defect in IE having CQ Bridge
				// In RQM, request to /jazz/web/projects/Quality%20Manager#action=com.ibm.rational.cm.showRecord...
				// will get redirected to /jazz/web/console/Quality%20Manager with parameters after hash(#) being lost
				// because of IE bug (e.g. http://www.ianhoar.com/2008/11/16/internet-explorer-6-and-redirected-anchor-links/)
				// so send request to /jazz/web/console/
				var rootPath = net.jazz.ajax._contextRoot + net.jazz.ajax._webuiPrefix;
				var projectsPath = "projects/";
				if(winUrl.path && winUrl.path.indexOf(rootPath) == 0){
					var index = winUrl.path.indexOf("/", rootPath.length);
					if(index != -1){
						projectsPath = winUrl.path.substring(rootPath.length, index+1);
					}
				}

				baseUrl = urlRoot + rootPath + projectsPath + encodeURIComponent(getCurrentProjectArea().name) + window.location.search;
			}else{
				baseUrl = "";
			}
			return baseUrl + "#action="+this.ACTION_PREFIX+info.action+"&id="+encodeURIComponent(id)+"&type="+type+"&title="+encodeURIComponent(label || "");
		}else{
			return "";
		}
	},

	getResourceHref: function(type, id){
		if(type == "RECORD" && id.match(/.+:.+\/.+@.+\/.+/i)){
			// id example: cq.repo.cq-record:16777224/33554440@7.0.0/SAMPL
			// URL: http://hostname:port/rcm/repo/7.0.0/db/SAMPL/record/16777224-33554440
			var temp = id.substring(id.indexOf(":")+1);
			var sep = temp.indexOf("@");
			var typeAndId = temp.substring(0, sep);
			var sep2 = typeAndId.indexOf("/");
			var recordTypeId = typeAndId.substring(0, sep2); 
			var recordId = typeAndId.substring(sep2+1); 
			var dbAndSchema = temp.substring(sep+1);
			var sep3 = dbAndSchema.indexOf("/");
			var schemaId = dbAndSchema.substring(0, sep3);
			var dbId = dbAndSchema.substring(sep3+1);
			return this._getBaseUrl() + "/repo/" + schemaId + "/db/" + dbId + "/record/" + recordTypeId + "-" + recordId;
		}
		return this.getActionHref(type, id, "");
	},
	
	_getBaseUrl: function(){
		// The URL _must_ be absolute.
		// FIXME: Need a dojo.require() statement to refer to net.jazz.ajax??
		return window.location.protocol + "//" + window.location.host.toLowerCase() + net.jazz.ajax._contextRoot + this.CM_SERVICE_CONTEXT_ROOT; 
	},
	
	/*
	 * Gets icon URL for a given artifact type
	 * type : String
	 * 	it takes either value of 
	 * remote : boolean (optional)
	 * 	pass true if it is a remotely-mastered icon
	 */
	getIconUrl: function(/* String */ type, /* boolean */ remote){
		var info = TypeInfo[type];
		if(info){
			var url = (remote && info.remoteUrl) ? info.remoteUrl : info.iconUrl;
			return dojo.moduleUrl("com.ibm.rational.cm.web", url);
		}else{
			return "";
		}
	},
	
	convertResourceURLtoPresentationURL: function(type, resourceURL){
		var id = this.getResourceIdFromURL(type, resourceURL);
		if(id){
			return this.getActionHref(type, id, "", true);
		}
		return resourceURL;
	},
	
	getResourceIdFromURL: function(type, resourceURL){
		if(type == "RECORD" && resourceURL.match(/.+\/repo\/.+\/db\/.+\/record\/.+-.+/i)){
			// resouce id must be format like http://hostname:port/rcm/repo/7.0.0/db/SAMPL/record/16777224-33554440
			var record = this._getResourcePart(resourceURL, "record");
			var sep = record.indexOf("-");
			var typeId = record.substring(0, sep);
			var recordId = record.substring(sep+1);
			var id = "cq.repo.cq-record:" + typeId + "/" + recordId +
									"@" + this._getResourcePart(resourceURL, "repo") +
									"/" + this._getResourcePart(resourceURL, "db");
			return id;
		} else if(type == "RECORD-TYPE" && resourceURL.match(/.+\/repo\/.+\/db\/.+\/record-type\/.+/i)){
			// /repo/7.0.0/db/SAMPL/record-type/16777899
			var typeId = this._getResourcePart(resourceURL, "record-type");
			var id = "cq.repo.cq-rectype:" + typeId +
									"@" + this._getResourcePart(resourceURL, "repo") +
									"/" + this._getResourcePart(resourceURL, "db");
			return id;
		}
		return null;
	},
	
	_getResourcePart: function(resourceURL, part){
		var i = resourceURL.indexOf("/" + part + "/");
		if(i > 0){
			var temp = resourceURL.substring(i+part.length+2);
			var end = temp.indexOf("/");
			if(part == "record") end = temp.substring(end+1).indexOf("/"); // special case
			return end < 0 ? temp : temp.substring(0, end);
		}
		return "";
	}
});

})();