/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo");//$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$

(function() { // module container

var OPEN= 1;
var DONE= 2;
var INPROGRESS= 4;

var ITEM_TYPES				= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES;
	
com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo= {
		
	_stateGroupMap: null,
	_stateActionsMap: null,
	
	// ---- API ------------------------------------------------------------------------------------------------------------
	
	getId: function() {
		return this.itemId;
	},
	
	getStartStateId: function() {
		return this.todoStates[0];
	},
	
	getStateIds: function() {
		var result= [];
		for ( var i = 0; i < this.todoStates.length; i++) {
			var state= this.todoStates[i];
			result.push(state);
		}
		for (i = 0; i < this.progressStates.length; i++) {
			state= this.progressStates[i];
			result.push(state);
		}
		for (i = 0; i < this.doneStates.length; i++) {
			state= this.doneStates[i];
			result.push(state);
		}
		for (i = 0; i < this.noneStates.length; i++) {
			state= this.noneStates[i];
			result.push(state);
		}
		return result;
	},
	
	getStateLabel: function(stateId) {
		var state= this._referencedItemStore.getItem(ITEM_TYPES.WORKFLOWSTATE, stateId);
		if (!state)
			return null;
		
		return state.label;
	},
	
	getStateIcon: function(stateId) {
		var state= this._referencedItemStore.getItem(ITEM_TYPES.WORKFLOWSTATE, stateId);
		if (!state)
			return null;
		
		return state.iconURL;
	},

	getStateGroup: function(stateId) {
		this._ensureStateGroupMapsCreated();
		
		var result= this._stateGroupMap[stateId];
		if (result)
			return result;
		
		return "unknown"; //$NON-NLS-1$
	},
	
	getActions: function(planItem) {
		this._ensureActionMapsCreated();
		
		var actions= this._stateActionsMap[planItem.getState().getId()];
		if (!actions) {
			return [];
		} else {
			return actions;
		}
	},
	
	getWorkflowAction: function(actionId) {
		for ( var i = 0; i < this.workflowActions.length; i++) {
			var action= this.workflowActions[i];
			if (action.actionId == actionId)
				return action;
		}
		
		return null;
	},
	
	getActionResultState: function(actionId) {
		var action=this.getWorkflowAction(actionId);
		if (!action)
			return null;
		
		return action.endState;
	},
	
	getAction: function(planItem, targetStateGroupId) {
		this._ensureActionMapsCreated();
		
		var actions= this._stateActionsMap[planItem.getState().getId()];
		if (!actions)
			return null;
		
		this._ensureStateGroupMapsCreated();
		
		for ( var i = 0; i < actions.length; i++) {
			var action= actions[i];
			
			var resultGroup= this._stateGroupMap[action.endState];
			if (resultGroup == targetStateGroupId)
				return action;
		}
		
		return null;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_ensureStateGroupMapsCreated: function() {
		if (this._stateGroupMap)
			return;
		
		this._stateGroupMap= {};
		for ( var i = 0; i < this.todoStates.length; i++) {
			var state= this.todoStates[i];
			this._stateGroupMap[state]= OPEN; //$NON-NLS-1$
		}
		for (i = 0; i < this.progressStates.length; i++) {
			state= this.progressStates[i];
			this._stateGroupMap[state]= INPROGRESS; //$NON-NLS-1$
		}
		for (i = 0; i < this.doneStates.length; i++) {
			state= this.doneStates[i];
			this._stateGroupMap[state]= DONE; //$NON-NLS-1$
		}
	},
	
	_ensureActionMapsCreated: function() {
		if (this._stateActionsMap)
			return;
		
		this._stateActionsMap= {};
		for ( var i = 0; i < this.workflowActions.length; i++) {
			var action= this.workflowActions[i];
			
			for ( var j = 0; j < action.startStates.length; j++) {
				var state= action.startStates[j];
				
				var stateActions= this._stateActionsMap[state];
				if (!stateActions) {
					stateActions= new Array();
					this._stateActionsMap[state]= stateActions;
				}
				stateActions.push(action);
			}
		}
	},

	__sentinel: null // terminates this class definition
};

com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo.OPEN= OPEN;
com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo.DONE= DONE;
com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo.INPROGRESS= INPROGRESS;

})();
