/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.rtc.web.ui.internal.ProcessInitializer"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.requireLocalization("com.ibm.team.rtc.web.ui.internal", "ProcessInitializerMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var getAuthenticatedUserId = com.ibm.team.repository.web.client.session.getAuthenticatedUserId;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var messages = dojo.i18n.getLocalization("com.ibm.team.rtc.web.ui.internal", "ProcessInitializerMessages");
var SERVICE_URI = "com.ibm.team.process.internal.common.service.IProcessRestService";
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

com.ibm.team.rtc.web.ui.internal.ProcessInitializer = {
	// summary:
	//		Adds process awareness to Jazz Ajax Foundation web UIs.  
	//			- Reads project areas from InitializationData and sets global vars.  
	//
	// description:
	//		1) Implement jazz.app._JazzApplication.
	//		2) Call 'init', passing the _JazzApplication context as the first parameter.
	//			eg. com.ibm.team.process.web.ui.ApplicationInitializer.init(this);
	// prereq: 
	//		Repository and InitializationData must be initialized. Call 'init' 
	//		on 'com.ibm.team.repository.web.ui.ApplicationInitializer'.
	
	init: function (application) {
		// init the project area global variables, and projects to context switcher
		var pathname = window.location.pathname;
		var projectName = pathname.substring(pathname.lastIndexOf('/')+1); //$NON-NLS-1;
		var decodedName = decodeURIComponent(projectName);
		
		var projectArea = dojo.setObject("com.ibm.team.process.web.projectArea", {}); //$NON-NLS-1$
		try {
			var projectAreas = InitializationData.get("com.ibm.team.process.service.projectAreaWebUIInitializionData").projectAreas; //$NON-NLS-1$
			if (projectName) {
				//if we found a project name from the URL, match with project from server
				for (var i = 0; i < projectAreas.length; i++) {
					if (projectAreas[i].name == decodedName) {
						projectArea.itemId = projectAreas[i].itemId;
						projectArea.name = projectAreas[i].name;
						projectArea.archived = projectAreas[i].archived;
					}
				}
			}
		} catch (e) {
			console.error(e);
			var redirect = true;
		}
		
		if (decodedName !== "web" && (!projectArea.itemId || pathname.indexOf("/web/projects/") < 0 || redirect)) {
			// redirect to '/web' if the URI isn't well-formed or projects are missing.
			var href = net.jazz.ajax._contextRoot + "/web";
			if (window.location.search) {
				href += window.location.search;
			}
			window.location.href = href;
		} else {
			var contextArgs = {
				contexts: projectAreas,
				onChange: this._selectContext,
				selectLabel: messages.selectProject,
				emptyLabel: messages.noProjects,
				contextProvider: dojo.hitch(this,this._getOtherProjectAreas)
			}; 
			if (decodedName === "web") {
				// if we're at '/web' show the project list page.
				application.config.usePageRegistry = false;
				if (projectAreas) {
					application.config.navbarItems.push({
						label: messages.projectAreas,
						pageId: "com.ibm.team.process.projectAreaList",
						isDefault: true
					});
				}
			} else {
				contextArgs.currentContext = projectArea;
			}
			application.config.contextSwitcher = contextArgs;
		}
		return contextArgs;
	},
	
	_getOtherProjectAreas: function (dfd) {
		var srh = new ServiceResponseHandler(dfd, "callback", "errback");
		var authContribId = getAuthenticatedUserId();
		var serviceRequest = new ServiceRequest(SERVICE_URI, "getAllProjectAreas", {hideArchivedProjects: true, userId: authContribId});
		TeamServerClient.invokeService(serviceRequest, srh);
	},
	_selectContext: function(contextObj) {
		var curUrl = new dojo._Url(location.href);
		
		if (curUrl.query) {  //check for debug mode
			var query = "?" + curUrl.query;	
		} else {
			var query = "";
		}

		var curPageId = PlatformUI.getWorkbench().currentPageId;
		if (curPageId) {
			var action = "#action=jazz.viewPage&id=" + curPageId;
		} else {
			var action = "";
		}

		var href = "";
		if (contextObj.admin) {
			href = net.jazz.ajax._contextRoot + "/admin" + query;
		} else {
			var encodedName = encodeURIComponent(contextObj.name);
			href = net.jazz.ajax._contextRoot + "/web/projects/" + encodedName + query + action;
		}
		return href;
	}
}
})();