/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.data.dom"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	dojo.require("jazz.app.ResourceLink");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ResourceLink = jazz.app.ResourceLink;
	
	function setText(e, text){
		e.appendChild(document.createTextNode(text));
	}
	
	var PROJECT_ACTION = "com.ibm.team.process.editProjectArea"; //$NON-NLS-1$
	var PROJECT_ACTION_PREFIX = PROJECT_ACTION + "&itemId="; //$NON-NLS-1$
	var CONTRIBUTORS_IN_PROJECT_ACTION = "#action=com.ibm.team.process.usersInProjectArea&projectAreaItemId="; //$NON-NLS-1$
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas", com.ibm.team.repository.web.ui.internal.admin.ItemList, { //$NON-NLS-1$
		constructor: function(){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
			this._actionToolbars = [];
			var hash = window.location.hash.toLowerCase();
			var index = hash.indexOf("find251users=true"); //$NON-NLS-1$
			if (index > -1) {
				this.findUsers = true;
			}
			setText(this.heading, this.messages.ManageProjectAreas_1);
			setText(this.description, this.messages.ManageProjectAreas_2);
			dojo.addClass(this.tableContents, ProcessConstants.CSSClass.TOOLBAR_TABLE);
			this.tableHeading.parentNode.removeChild(this.tableHeading);
			this.setHeaderRow([this.messages.ManageProjectAreas_3, this.messages.ManageProjectAreas_4, this.messages.ManageProjectAreas_ActionsColumnHeader]);
			this.headerRow.cells[0].style.width = "30%"; //$NON-NLS-1$
			dojo.addClass(this.headerRow.cells[2], ProcessConstants.CSSClass.ACTION_COLUMN);
			this.addNewButton(PROJECT_ACTION_PREFIX + "new", this.messages.ManageProjectAreas_5); //$NON-NLS-1$
			this.refresh();
		},
		
		refresh: function(){
			this.clearState();
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var td = document.createElement("td"); //$NON-NLS-1$
			td.colSpan = 2;
			td.appendChild(document.createTextNode(this.messages.ManageProjectAreas_6));
			tr.appendChild(td);
			this.tbody.appendChild(tr);
			var srh = new ServiceResponseHandler(this, "showProjects", "handleError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getAllProjectAreas(srh, {
				hideArchivedProjects: true
			});
		},
		
		clearState: function(){
			dojox.data.dom.removeChildren(this.tbody);
			dojox.data.dom.removeChildren(this.messageDiv);
			dojox.data.dom.removeChildren(this.messageBox);
			this.messageBox.className = ""; //$NON-NLS-1$
			this._destroyActionToolbars();
			
		},
		
		showProjects: function(data){
			dojox.data.dom.removeChildren(this.tbody);
			if (data != null) {
				var sortedData = data.sort(function(a, b){
					return a.name.toLowerCase().localeCompare(b.name.toLowerCase());
				});
				for (var i = 0; i < sortedData.length; i++) {
					var actionHref = PROJECT_ACTION_PREFIX + sortedData[i].itemId;
					var descNode = this._createDescriptionNode(sortedData[i].summary, sortedData[i].description);
					var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
					this._actionToolbars.push(trToolbar);
					
					var archiveAction = {};
					archiveAction.imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/archive_hov.gif"); //$NON-NLS-1$ //$NON-NLS-2$
					archiveAction.label = this.messages.ManageProjectAreas_7;
					archiveAction.callbackData = {
						itemId: sortedData[i].itemId,
						name: sortedData[i].name
					};
					var callback = function(e){ //$NON-NLS-1$
						dojo.stopEvent(e);
						if (confirm(this.messages.ManageProjectAreas_8)) {
							this.archiveProject(e.currentTarget.callbackData);
						}
					};
					trToolbar.addButton(this, callback, archiveAction);
					var row = this.addRow(actionHref, [sortedData[i].name, descNode, trToolbar.domNode]);
					
					var query = "a[href*='" + PROJECT_ACTION + "']";//$NON-NLS-1$ //$NON-NLS-2$
					var link = dojo.query(query, row)[0];
					var presentationUri = link.href;
					link.href = this.getProjectAreaUrl(sortedData[i].itemId);
					var rl = new ResourceLink({
						'presentationUri': presentationUri //$NON-NLS-1$ 
					}, link);
					
					if (this.findUsers) {
						this._addFindUsersButton(trToolbar, sortedData[i], "users-in-project", this.messages.ManageProjectAreas_13, false); //$NON-NLS-1$
						this._addFindUsersButton(trToolbar, sortedData[i], "users-not-in-project", this.messages.ManageProjectAreas_14, true); //$NON-NLS-1$
					}
				}
			} else {
				this.messageDiv.appendChild(document.createTextNode(this.messages.ManageProjectAreas_9));
			}
		},
		
		getProjectAreaUrl: function(projectAreaItemId){
			var location = new String(document.location);
			var path = location.split("/");//$NON-NLS-1$
			var serverAddress;
			if (path.length > 0) {
				serverAddress = path.shift() + "//";//$NON-NLS-1$
				path.shift(); //empty from the double //
				serverAddress = serverAddress + path.shift();
			}
			return serverAddress + net.jazz.ajax._contextRoot + "/process/project-areas/" + projectAreaItemId; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_createDescriptionNode: function(summary, description){
			if (dojo.isIE && summary !== undefined && summary.length === 0 &&
			description !== undefined &&
			description.length === 0) {
				// IE needs a space or it won't paint the TD
				summary = " "; //$NON-NLS-1$
			}
			var descNode = document.createElement("div"); //$NON-NLS-1$
			descNode.appendChild(document.createTextNode(summary));
			if (description) {
				if (summary) {
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
				}
				descNode.appendChild(document.createTextNode(description));
			}
			return descNode;
		},
		
		_addFindUsersButton: function(toolbar, projectArea, imageName, label, notInProject){
		
			var action = {};
			var imagePath = "ui/internal/admin/projects/images/" + imageName + ".gif"; //$NON-NLS-1$ //$NON-NLS-2$
			action.imgSrc = dojo.moduleUrl("com.ibm.team.process.web", imagePath); //$NON-NLS-1$ 
			action.label = label;
			action.callbackData = {
				actionHref: CONTRIBUTORS_IN_PROJECT_ACTION + projectArea.itemId + "&projectAreaName=" + projectArea.name + "&notInProject=" + notInProject //$NON-NLS-1$ //$NON-NLS-2$
			};
			var callback = function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				window.location.hash = e.currentTarget.callbackData.actionHref;
			};
			toolbar.addButton(this, callback, action);
		},
		
		archiveProject: function(projectArea){
			if (!this._projectAreaArchiving) {
				this._projectAreaArchiving = projectArea;
				this._disableToolbars(true);
				var srh = new ServiceResponseHandler(this, "archiveCallback", "archiveErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
				this.displayPendingMessage(this.messages.ManageProjectAreas_10);
				ProcessClient.postArchiveProjectArea({
					uuid: projectArea.itemId
				}, srh);
			}
		},
		
		archiveCallback: function(){
			dojo.publish(ProcessConstants.ProjectSelectorContext.DELETE, [this._projectAreaArchiving.name]);
			delete this._projectAreaArchiving;
			this.refresh();
			this.displaySuccessMessage(this.messages.ManageProjectAreas_11);
		},
		
		archiveErrorCallback: function(err){
			delete this._projectAreaArchiving;
			this.displayErrorMessage(dojo.string.substitute(this.messages.ManageProjectAreas_12, [err.message]));
			this._disableToolbars(false);
		},
		
		_disableToolbars: function(disabled){
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].disableToolbar(disabled);
			}
		},
		
		_destroyActionToolbars: function(){
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].destroyRecursive();
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function(){
			this._destroyActionToolbars();
			
			this.inherited(arguments);
			
		}
	});
	
})();
