/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var ContextCSSClass = {
		PROJECT: 'projectConfigIcon', //$NON-NLS-1$
		TEAM: 'teamConfigIcon', //$NON-NLS-1$
		TIMELINES: 'timelinesConfigIcon', //$NON-NLS-1$
		TIMELINE: 'timelineConfigIcon', //$NON-NLS-1$
		PROJECT_TIMELINE: 'projectTimelineConfigIcon', //$NON-NLS-1$
		ITERATION_TYPES: 'iterationTypesConfigIcon', //$NON-NLS-1$
		ITERATION_TYPE: 'iterationTypeConfigIcon', //$NON-NLS-1$
		ITERATIONS: 'iterationConfigIcon', //$NON-NLS-1$
		ITERATION: 'iterationConfigIcon', //$NON-NLS-1$
		CURRENT_ITERATION: 'currentIterationConfigIcon', //$NON-NLS-1$
		CURRENT_ITERATION_LABEL: 'currentIterationLabel', //$NON-NLS-1$
		COMPLETED_ITERATION: 'completedIterationConfigIcon', //$NON-NLS-1$
		PERMISSIONS: 'permissionsConfigIcon' //$NON-NLS-1$
	};
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeTree", [dijit.Tree], { //$NON-NLS-1$
		labelAttr: 'name', //$NON-NLS-1$
		childrenAttr: ['items'], //$NON-NLS-1$
		showRoot: false,
		persist: false,
		/*
		 * Constructs the Timeline tree instance
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    model: the model the tree should use for it's data
		 *    editorContext: the editor context tree is being used in... a value in ProcessConstants.EditorContext
		 *    onClick: the function to call when a node in the tree is clicked on
		 *  }
		 */
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		getLabel: function(item){
			var label = this._getDisplayLabel(item);
			if (item.enabledContext == ProcessConstants.PermissionContext.DEV_LINE) {
				if (item.projectTimeline == "true") { //$NON-NLS-1$
					label = dojo.string.substitute(this.messages.PermissionsScopeTree_TimelineDecoratorText, [label, this.messages.PermissionsScopeTree_ProjectTimelineDecoratorText]);
				}
			}
			return label;
		},
		
		
		_getDisplayLabel: function(item){
			if (item && !item.root) {
				var store = this.model.store;
				var name = dojo.trim(store.getValue(item, "name")); //$NON-NLS-1$
				if (!name || name.length === 0) {
					name = store.getValue(item, "contextId"); //$NON-NLS-1$
					if (store.isIteration(item)) {
						name = this.getIterationId(name);
					}
				}
				return name;
			}
		},
		
		getIterationId: function(contextId){
			var segments = contextId.split(ProcessConstants.PERMISSION_PATH_SEPARATOR);
			return segments[segments.length - 1];
		},
		
		getLabelClass: function(item, opened){
			var store = this.model.store;
			if (item.enabledContext == ProcessConstants.PermissionContext.ITERATION) {
				if (store.hasAttribute(item, "current") && store.getValue(item, "current") === true) { //$NON-NLS-1$ //$NON-NLS-2$
					if (store.hasAttribute(item, "leaf") && store.getValue(item, "leaf") === true) { //$NON-NLS-1$ //$NON-NLS-2$
						return ContextCSSClass.CURRENT_ITERATION_LABEL;
					}
				}
			}
		},
		
		getIconClass: function(item){
			if (item && !item.root) {
				var store = this.model.store;
				var id = this.model.getIdentity(item);
				if (id === ProcessConstants.PermissionsScopeContext.PROJECT) {
					return ContextCSSClass.PROJECT;
				} else if (id === ProcessConstants.PermissionsScopeContext.TEAM) {
					if (this.editorContext === ProcessConstants.EditorContext.PROJECT) {
						return ContextCSSClass.TEAM;
					} else {
						return ContextCSSClass.PERMISSIONS;
					}
				} else if (id === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES) {
					return ContextCSSClass.ITERATION_TYPES;
				} else if (id === ProcessConstants.PermissionsScopeContext.DEV_LINES) {
					return ContextCSSClass.TIMELINES;
				} else if (id === ProcessConstants.PermissionsScopeContext.ITERATIONS) {
					return ContextCSSClass.ITERATION;
				}
				
				if (store.hasAttribute(item, "enabledContext")) { //$NON-NLS-1$
					var context = store.getValue(item, "enabledContext"); //$NON-NLS-1$
					if (context === ProcessConstants.PermissionContext.ITERATION_TYPE) {
						return ContextCSSClass.ITERATION_TYPE;
					} else if (context === ProcessConstants.PermissionContext.DEV_LINE) {
						if (store.hasAttribute(item, "projectTimeline") && store.getValue(item, "projectTimeline") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							return ContextCSSClass.PROJECT_TIMELINE;
						} else {
							return ContextCSSClass.TIMELINE;
						}
					} else if (context === ProcessConstants.PermissionContext.ITERATION) {
						if (store.hasAttribute(item, "current") && store.getValue(item, "current") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							return ContextCSSClass.CURRENT_ITERATION;
						} else if (store.hasAttribute(item, "completed") && store.getValue(item, "completed") === true) { //$NON-NLS-1$ //$NON-NLS-2$
							return ContextCSSClass.COMPLETED_ITERATION;
						} else {
							return ContextCSSClass.ITERATION;
						}
					}
				}
			}
			
		},
		
		focusItem: function(item){
			var newTimelineNode = this._itemNodeMap[this.model.getIdentity(item)];
			this.focusNode(newTimelineNode);
			
		},
		
		// From tree.js
		_onBlur: function(){
			if (this.lastFocused) {
				var labelNode = this.lastFocused.labelNode;
				// Process related additions
				// we want to see what was last focused in the tree
				// dojo.removeClass(labelNode, "dijitTreeLabelFocused");	
			}
		},
		
		// From tree.js for workitem 68440
		_onClick: function(/*Event*/e){
			// summary: translates click events into commands for the controller to process
			var domElement = e.target;
			
			// find node
			var nodeWidget = dijit.getEnclosingWidget(domElement);
			if (!nodeWidget || !nodeWidget.isTreeNode) {
				return;
			}
			
			if ((this.openOnClick && nodeWidget.isExpandable) ||
			(domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText)) {
				// expando node was clicked, or label of a folder node was clicked; open it
				if (nodeWidget.isExpandable) {
					this._onExpandoClick({
						node: nodeWidget
					});
				}
			} else {
				this._publish("execute", { //$NON-NLS-1$
					item: nodeWidget.item,
					node: nodeWidget
				});
				// Had to swap the following two lines in execution order for workitem 68440
				this.focusNode(nodeWidget);
				this.onClick(nodeWidget.item, nodeWidget);
			}
			dojo.stopEvent(e);
		},
		
		focusNode: function(/* _tree.Node */node){ // from Tree.js
			// set focus so that the label will be voiced using screen readers
			node.labelNode.focus();
			// Process related additions
			this._focusChanged(node);
		},
		
		_focusChanged: function(node){
			// Should be overriden to perform the appropriate
			// action 
		},
		
		mayHaveChildren: function(item){
			var id = this.model.getIdentity(item);
			if (id === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES || id === ProcessConstants.PermissionsScopeContext.DEV_LINES || id === ProcessConstants.PermissionsScopeContext.ITERATIONS) {
				return true;
			}
			
			return false;
		},
		
		uninitialize: function(){
			delete this.model;
			this.inherited(arguments);
		}
		
	});
})();
