/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$

(function() {

dojo.declare("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore", null, { //$NON-NLS-1$

	constructor: function(initialItems) {
		this._store= {};
		if (dojo.isArray(initialItems)) {
			this.addItems(initialItems);
		}
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	addItems: function(referencedItems) {
		for (var i= 0; referencedItems && i < referencedItems.length; i++) {
			var item= referencedItems[i];
			
			var typeStore= this._store[item.itemType];
			if (!typeStore)
				typeStore= this._store[item.itemType]= {}; 
			
			typeStore[item.itemId]= item;
		}
	},
	
	getItem: function(type, id) {
		var result= this._store[type] && this._store[type][id];
		if(result != null)
			return result;
			
		var subtypes= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.TYPE_HIERARCHY[type];
		for(var i= 0; subtypes && i < subtypes.length; i++) {
			result= this._store[subtypes[i]] && this._store[subtypes[i]][id];
			if(result != null)
				return result;
		}
		
		return null;
	},

	getItems: function(type) {
		var result= [];
		var store= this._store[type] || {};
		for (var id in store) {
			result.push(store[id]);
		}
		return result;
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES= {
	PLANRECORD: "item:com.ibm.team.apt:IterationPlanRecord", //$NON-NLS-1$
	CONTRIBUTOR: "item:com.ibm.team.repository:Contributor", //$NON-NLS-1$
	PROCESSAREA: "item:com.ibm.team.process:ProcessArea", //$NON-NLS-1$
	PROJECTAREA: "item:com.ibm.team.process:ProjectArea", //$NON-NLS-1$
	TEAMAREA: "item:com.ibm.team.process:TeamArea", //$NON-NLS-1$
	ITERATION: "item:com.ibm.team.process:Iteration", //$NON-NLS-1$
	DEVLINE: "item:com.ibm.team.process:DevelopmentLine", //$NON-NLS-1$

	CATEGORY: "item:com.ibm.team.workitem:Category", //$NON-NLS-1$
	WORKITEM: "item:com.ibm.team.workitem:WorkItem", //$NON-NLS-1$
	ATTRIBUTE: "item:com.ibm.team.workitem:Attribute", //$NON-NLS-1$
	WITYPE: "object:com.ibm.team.workitem:type", //$NON-NLS-1$
	WORKFLOW: "object:com.ibm.team.workitem:workflow", //$NON-NLS-1$
	WORKFLOWSTATE: "object:com.ibm.team.workitem:state", //$NON-NLS-1$
	
	CONTRIBUTORABSENCE: "item:com.ibm.team.apt.resource:ContributorAbsence", //$NON-NLS-1$
	WIKIPAGE: "item:com.ibm.team.foundation.wiki:WikiPage" //$NON-NLS-1$
};

com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.TYPE_HIERARCHY= {
	"item:com.ibm.team.process:ProcessArea": [ com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES.PROJECTAREA, com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES.TEAMAREA] //$NON-NLS-1$
};

})();
