﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

// TODO: Extend layoutObject?
class LegendEntry extends ChartObject {
	
	public static var EDGE_LENGTH = 10;

	private var _shape:IChartShape;
	
	private var _mcName:String;
	
	private static var _instanceCount:Number = 0;
	
	private static var _MC_PRE_NAME:String = "legend_entry_mc_";
	
	private static var _TEXT_FIELD_PRE_NAME:String = "legend_label_mc_";
	
	public static var TEXT_FORMAT:TextFormat = LegendEntry._initTextFieldFormat();

	public static var LABEL_OFFSET_X:Number = 20;

	private static var LABEL_OFFSET_Y:Number = -3;
	
	private static var _TEXT_FIELD_HEIGHT:Number = 16;
	
	
	public function LegendEntry(shape:IChartShape) {
		super();
		this._shape = shape;
		LegendEntry._instanceCount++;
	}

	private static function _initTextFieldFormat():TextFormat {
		var textFieldFormat = new TextFormat();
		textFieldFormat.color = 0x000000;
		textFieldFormat.font = ChartArea.FONT;
		// Each character
		// W X H
		// 6 X 7 pixels
		textFieldFormat.size = 10;  
		
		return textFieldFormat;
	}	
	
	public function draw(parentMovieClip:MovieClip, startX:Number, startY:Number, labelWidth:Number):Void {
		
		this._mcName = LegendEntry._MC_PRE_NAME + String(LegendEntry._instanceCount);
		
		this.mc = 
			parentMovieClip.createEmptyMovieClip(
				this._mcName, 
				parentMovieClip.getNextHighestDepth()
			);
		
		this.drawSwatch(startX, startY);
		this.drawLabel(startX + LegendEntry.LABEL_OFFSET_X, 
					   startY + LegendEntry.LABEL_OFFSET_Y,
					   labelWidth);	
		this._addListeners();
		
	}
	
	private function drawSwatch(startX:Number, startY:Number):Void {
		
		this.mc.beginFill(this._shape.getFillColour());
		this.mc.lineStyle(1, this._shape.getOutlineColour());
		this.mc.moveTo(startX, startY);
		this.mc.lineTo(startX + LegendEntry.EDGE_LENGTH, startY);
		this.mc.lineTo(startX + LegendEntry.EDGE_LENGTH, startY + LegendEntry.EDGE_LENGTH);
		this.mc.lineTo(startX, startY + LegendEntry.EDGE_LENGTH);
		this.mc.lineTo(startX, startY);
		this.mc.endFill();
	}	
	
	private function drawLabel(startX:Number, startY:Number, labelWidth:Number):Void {
		
		var textFieldName:String = LegendEntry._TEXT_FIELD_PRE_NAME + String(LegendEntry._instanceCount);
		
		this.mc.createTextField(
			textFieldName, 
			this.mc.getNextHighestDepth(), 
			startX, 
			startY, 
			labelWidth, 
			LegendEntry._TEXT_FIELD_HEIGHT
		);
		
		this.mc[textFieldName].text = TextUtility.truncateAndTrailOffText(this._shape.getLabel(), 
																		  LegendEntry.TEXT_FORMAT, 
																		  this.mc[textFieldName]._width);
		this.mc[textFieldName].setTextFormat(LegendEntry.TEXT_FORMAT);	
	}	
	
	private function _addListeners():Void {
		
		var context = this;
		
		this.mc.onRollOver = function() {
			this._alpha = 60;
			context._shape.doRollOver();
		};
		this.mc.onRollOut = function() {
			this._alpha = 100;
			context._shape.doRollOut();
		};

		var model:ChartModel = ChartEngine.getInstance().getModel();
		if (model.enableClickHandler) {
			this.mc.onPress = function() {
				context._shape.doDrillIn();
			}		
		}
	}
	
	public function destroy():Void {
		
		delete this._mcName;
		
		// Shape
		if (this._shape) {
			this._shape.destroy();
			delete this._shape;
		}
		// Instance count
		// TODO: If only some legend entries are destroyed and new ones created, chance of ID overlap ...
		LegendEntry._instanceCount--;
		
		super.destroy();
	}		
	
}