/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
/**
 * @author hyassine
 */
dojo.provide("com.ibm.team.scm.web.ui.internal.page.ScmPageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.team.scm.web.ui.internal.page.ScmPageController");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.StreamBrowse");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.ActiveTeamChanges");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.UserLocks");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.RepositoryWorkspaceBrowse");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.StreamWorkspaceView");
dojo.require("com.ibm.team.scm.web.ui.internal.view.browse.FilesFoldersView");
dojo.require("com.ibm.team.scm.web.ui.internal.view.common.Loading");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() {
	
	var checkNotNull		= net.jazz.ajax.toolbox.lang.checkNotNull;
	var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;
	
	var PlatformUI			= net.jazz.ajax.ui.PlatformUI;
	var ActionRegistry		= null;
	
	dojo.declare(
	"com.ibm.team.scm.web.ui.internal.page.ScmPageView",
	[dijit._Widget, dijit._Templated],
		{
			templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/page/templates/ScmPageView.html"),
			controller: null,
			
			displayError: function(message) {
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						message,
						"error");	
			},
			
			setHistoryPageSize: function(n) {
				this._streamWorkspaceView.setHistoryPageSize(n);
			},
			
			//Display the components of the corresponding stream selected for browsing.
			showStreamView: function(streamComponents) {
				checkNotNull(streamComponents, "streamComponents");
	
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						streamComponents,
						"stream");	
			},
			
			//Display the components of the corresponding workspace selected for browsing.
			showWorkspaceView: function(workspaceComponents) {
				checkNotNull(workspaceComponents, "workspaceComponents");
				
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						workspaceComponents,
						"workspace");	
			},
			
			//Display the selected item (file or folder).
			showElementView: function(subElements, elementType, tabArgs) {
				checkNotNull(elementType, "elementType");
				
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						subElements,
						elementType, tabArgs);	
			},
			
			//Display the component history (list of change sets).
			showComponentHistory: function(historyDTO) {
				checkNotNull(historyDTO, "historyDTO");
				
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						historyDTO,
						"componentHistory");
			},
			
			//Display the item history (list of change sets).
			showItemHistory: function(historyDTO) {
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						historyDTO,
						"itemHistory");
			},

			//Display the team activity (active team change sets).			
			showTeamChanges: function(teamChanges) {
				checkNotNull(teamChanges, "teamChanges");
				
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						teamChanges,
						"teamChanges");
			},
			
			//Display the change set.
			showChangeSetView: function(changeSetDTO, fromHistory) {
				checkNotNull(changeSetDTO, "changeSetDTO");
				if (fromHistory == "1") {
					this._setMainContent("StreamWorkspaceView", this._streamWorkspaceView, changeSetDTO, "changeSetFromHistory");
				}
				else {
					this._setMainContent("StreamWorkspaceView", this._streamWorkspaceView, changeSetDTO, "changeSetFromActivity");
				}
			},
			
			//Display the list of snapshots
			showSnapshots: function(snapshots) {
				checkNotNull(snapshots, "snapshots");
				this._setMainContent(
						"StreamWorkspaceView", 
						this._streamWorkspaceView, 
						snapshots, 
						"snapshots");
			},
			
			//Display a change set item (file or folder at a specific state).
			showStateContent: function(versionableDTO) {
				checkNotNull(versionableDTO, "versionableDTO");
				
				this._setMainContent(
						"StreamWorkspaceView",
						this._streamWorkspaceView,
						versionableDTO,
						"changeSetItem");
			},
			
			//Scroll the browser to the specified diff anchor.
			scrollToDiffInChangeset: function(diffId) {
				this._streamWorkspaceView.scrollToDiffInChangeset(diffId);
			},
			
			/*
			 * Required by Jazz web framework to make hyperlinking and back/forward
			 * work.
			 */
			getController: function() {
				return this.controller;
			},
			
			constructor: function(params) {
				this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			},
			/*
			 * Called by Dojo during widget creation lifecycle.
			 */
			postCreate: function() {
				this.controller = new com.ibm.team.scm.web.ui.internal.page.ScmPageController(this);
				
				this._streamWorkspaceView= null;
		 		this._loadingView= null;
				
				this._leftNavigationLayout = new jazz.ui.SideNavigationLayout({}, this._appContainer);
				
				//Lazily initialize the ActionRegistry alias
				ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			
				//Register actions
				ActionRegistry.registerAction("com.ibm.team.scm._showDefaultView", this.controller, "_showDefaultView");
				ActionRegistry.registerAction("com.ibm.team.scm.browseStream", this.controller, "browseStream");
				ActionRegistry.registerAction("com.ibm.team.scm.browseWorkspace", this.controller, "browseWorkspace");
				ActionRegistry.registerAction("com.ibm.team.scm.showTeamChanges", this.controller, "showTeamChanges");
				ActionRegistry.registerAction("com.ibm.team.scm.browseElement", this.controller, "browseElement");
				ActionRegistry.registerAction("com.ibm.team.scm.viewComponentHistory", this.controller, "viewComponentHistory");
				ActionRegistry.registerAction("com.ibm.team.scm.viewChangeSet", this.controller, "viewChangeSet");
				ActionRegistry.registerAction("com.ibm.team.scm.viewChangeSetItem", this.controller, "viewChangeSetItem");
				ActionRegistry.registerAction("com.ibm.team.scm.viewItemHistory", this.controller, "viewItemHistory");
				ActionRegistry.registerAction("com.ibm.team.scm.viewSnapshots", this.controller, "viewSnapshots");
				
				//Initialize the left-side navigation panel containing the lists of streams and workspaces.
				this._initializeNavigation();
				//Display the main page content.
				this._initializeContent();
			},
			
			/**
			 * Initializes the left navigation section with the list of streams in a project area, the list
			 * of repository workspaces for the logged in user, and access to view active teams changes.
			 */
			_initializeNavigation: function() {
				var args;
				
				this._streamBrowsingWidget = new com.ibm.team.scm.web.ui.internal.view.browse.StreamBrowse({controller: this.controller});
				var streamSection = this._leftNavigationLayout.addNavigationSection(this._streamBrowsingWidget, this.messages.ScmPageView_1 , true);
				this._streamBrowsingWidget.populate(streamSection);

				this._userLocksWidget = new com.ibm.team.scm.web.ui.internal.view.browse.UserLocks({controller: this.controller});
				var userLocksSection = this._leftNavigationLayout.addNavigationSection(this._userLocksWidget, this.messages.ScmPageView_2 , true);
				this._userLocksWidget.setNavigationSection(userLocksSection);
				this._userLocksWidget.refresh();
				
				this._repositoryWorkspaceWidget = new com.ibm.team.scm.web.ui.internal.view.browse.RepositoryWorkspaceBrowse({controller: this.controller});
				var workspaceSection = this._leftNavigationLayout.addNavigationSection(this._repositoryWorkspaceWidget, this.messages.ScmPageView_0, true);
				this._repositoryWorkspaceWidget.populate(workspaceSection);
				
				//this._activeTeamChangesWidget = new com.ibm.team.scm.web.ui.internal.view.browse.ActiveTeamChanges({controller: this.controller});
				//this._leftNavigationLayout.addNavigationSection(this._activeTeamChangesWidget, "Queries", true);
			},
			
			
			/**
			 * Initializes the main page content display.
			 */
			_initializeContent: function() {
				this._streamWorkspaceView = new com.ibm.team.scm.web.ui.internal.view.browse.StreamWorkspaceView({controller: this.controller});
			},
			
			/**
			 * Displays the source control main welcome page.
			 */
			_showMainPage: function() {
				this._streamBrowsingWidget.refreshSelection(-1);
				this._repositoryWorkspaceWidget.refreshSelection(-1);
				//isMainPage to indicate that the page to be displayed is the welcome page.
				this._streamWorkspaceView.setContent({isMainPage: true});
				this._setMainContent("StreamWorkspaceView", this._streamWorkspaceView);	
			},
			
			/**
			 * Sets the content to be displayed.
			 * 		- List of components.
			 * 		- Folders and files under a component or folder.
			 * 		- File content to be viewed.
			 * 		- Component history (changer-sets).
			 * 		- Item history (change sets).
			 * 		- Change sets
			 * 		- Change set item (file or folder at a specific state).
			 */
			_setMainContent: function(widgetId, mainAreaWidget, subElements, elementType, tabArgs) {
				this._currentMainWidgetId= widgetId;
				this._currentMainWidget= mainAreaWidget;
				
				//This is always executed unless _showMainPage makes the call to _setMainContent
				if(elementType) {
				    if(elementType=="error") {
				        mainAreaWidget.scrollToTop();
				        mainAreaWidget.displayErrorMessage(subElements);
				    }
				    
					else if(elementType=="stream") {
						//Unselect a workspace in case it was selected.
						this._repositoryWorkspaceWidget.refreshSelection("-1");
						//Highlight the selected stream.
						this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isStream: true}, subElements);
					}
					
					else if(elementType=="workspace") {
						//Unselect a stream in case it was selected.
						this._streamBrowsingWidget.refreshSelection("-1");
						//Highlight the selected workspace.
						this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isWorkspace: true}, subElements);
					}
					
					else if(elementType=="component") {
						//Highlight higher level workspace/stream selection.
						if(subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}
						mainAreaWidget.scrollToTop();
						var contentArgs = tabArgs ? tabArgs : {};
						contentArgs.isComponent = true;
						mainAreaWidget.setContent(contentArgs, subElements);
					}
					//Item - File or Folder
					else if(elementType=="item"){
						//Highlight higher level workspace/stream selection.
						if(subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}
						mainAreaWidget.scrollToTop();
						var contentArgs = tabArgs ? tabArgs : {};
						contentArgs.isOther = true;
						mainAreaWidget.setContent(contentArgs, subElements);
					}
					
					else if(elementType=="componentHistory") {
						//Highlight higher level workspace/stream selection.
						if(subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isComponentHistory: true}, subElements);
					}
					
					else if(elementType=="itemHistory") {
						//Highlight higher level workspace/stream selection.
						if(subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isItemHistory: true}, subElements);
					}
					
					else if(elementType=="teamChanges") {
						//Unhighlight any workspace/stream selection.
						this._repositoryWorkspaceWidget.refreshSelection("-1");
						this._streamBrowsingWidget.refreshSelection("-1");
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isTeamChanges: true}, subElements);
					}
					
					else if(elementType=="snapshots") {
						/* //Will have to add once a DTO is sent vs a list
						if(subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}*/
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isSnapshots: true}, subElements);
					}
					
					else if(elementType=="changeSetFromHistory" || elementType=="changeSetFromActivity") {
						//Change set accessed from the history view or linked from a work item
						if (elementType == "changeSetFromHistory") {
						    if (subElements.workspace != null) {
							    if (subElements.workspace.stream == "true" || subElements.workspace.stream == true) {
							    	this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							    	this._repositoryWorkspaceWidget.refreshSelection("-1");
							    }
							    else {
							    	this._streamBrowsingWidget.refreshSelection("-1");
							    	this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
							    }
							}
							mainAreaWidget.scrollToTop();
							mainAreaWidget.setContent({isChangeSetFromHistory: true}, subElements);
						}
						//Change set accessed from the team activity view.
						else if (elementType == "changeSetFromActivity") {
							this._repositoryWorkspaceWidget.refreshSelection("-1");
							this._streamBrowsingWidget.refreshSelection("-1");
							mainAreaWidget.scrollToTop();
							mainAreaWidget.setContent({isChangeSetFromActivity: true}, subElements);
						}	
					}
					
					else if(elementType=="changeSetItem") {
						//Highlight higher level workspace/stream selection.
						var ws = subElements.workspace;
						if (ws != null) {
						if(ws.stream == "true" || ws.stream == true) {
							this._streamBrowsingWidget.refreshSelection(subElements.workspace.itemId);
							this._repositoryWorkspaceWidget.refreshSelection("-1");
						}
						else {
							this._streamBrowsingWidget.refreshSelection("-1");
							this._repositoryWorkspaceWidget.refreshSelection(subElements.workspace.itemId);
						}
						}
						mainAreaWidget.scrollToTop();
						mainAreaWidget.setContent({isChangeSetItem: true}, subElements);			
					}
				}

				this._leftNavigationLayout.setMainContent(mainAreaWidget.domNode);
			}
			
		}
	);
	
}) ();
