/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var EditorUtil				= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;

// delay between user input and running a search (in milliseconds)
var DELAY					= 750;

// cap max number of query results at 50
var MAX						= 50;

var KEY_RETURN			= 13;
var KEY_ENTER			= 14;
var KEY_SPACE			= 32;
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/SelectWorkItemContent.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/SelectWorkItemContent.css"),
		constructor: function(params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.labelEnterSearchString = this.messages.labelEnterSearchString;
			this.buttonOK = this.messages.buttonOK;
			
			this.buttonCancel = this.messages.buttonCancel;
			this.htmlEnterSearchString = this.messages.labelEnterSearchString;
		},
		postCreate: function() {
			// mark that keypresses from this control should be ignored w/r/t
			// dirtying the editor
			this.queryInput.__jazzIgnoreKeystroke= true;
			var nameValueLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout();
			this._childWidgets = [ nameValueLayout ];
			this.projectAreaSelectDiv = document.createElement("span");
			this.projectAreaSelectDiv.className = "selectInput";
			var projLabel =  this.messages.labelProjectArea; 
			nameValueLayout.addRow( {labelText: projLabel, valueNode: this.projectAreaSelectDiv });
			this.typeSelectDiv = document.createElement("span");
			this.typeSelectDiv.className = "selectInput";
			var typeLabel =  this.messages.labelTypeFilter;
			nameValueLayout.addRow({labelText: typeLabel, valueNode: this.typeSelectDiv });
			this.nameValueSection.appendChild(nameValueLayout.domNode);
			this._initProjectAreaSelector();
			this._projectSelect.tabindex = 5;
			// Default to show recently viewed if it is unspecified but we are in the work item context
			if(this.showRecentlyViewed == null) {
				if(this.parentController && this.parentController.pageController 
					&& this.parentController.pageController._workItemPageView 
					&& this.parentController.pageController._workItemPageView._recentWorkItemWidget) {
					this.showRecentlyViewed = true;
				} else {
					this.showRecentlyViewed = false;
				}
			} 

			if(this.showRecentlyViewed) {
				this.resultTitle.innerHTML = this.messages.labelRecentlyViewed;
			} else {
				this.resultTitle.innerHTML = this.messages.labelMatching;
			}
			
			this._connectEvents();
		},
		destroy: function() {
			dojo.forEach(this._childWidgets, function(widget) { widget.destroy() } );
		},
		_initProjectAreaSelector: function() {
			this._currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
			this._projectSelect = document.createElement("select");
			this._projectSelect.className = "selectSelect";
			/*
			 * If the projectAreaItemId is provide, use that, don't show current
			 */
			if(!this.projectAreaItemId ||  this._currentProjectArea.itemId ==  this.projectAreaItemId) {
				var option = EditorUtil._addOption(this._projectSelect, new Option(this._currentProjectArea.name, this._currentProjectArea.itemId), true);				
			}

			this.projectAreaSelectDiv.appendChild(this._projectSelect);
			
			var srh= new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_setProjectAreaOptions", "_handleError");
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getProcessAreaHierarchies(srh);
		},
		
		_handleError: function(e) {
			console.debug("Error loading project areas. Option ignored."); // $NON-NLS-1
		},
		_setProjectAreaOptions: function(projectAreaTree) {
			var projectAreas = [];
			this._collectProcessAreas(projectAreaTree, projectAreas, "", false, false);
			projectAreas.sort(function(a, b){
				try{
					return a.label.toLowerCase() > b.label.toLowerCase() ? 1 : -1;
				} catch(e) { 
					return -1 
				}
			});
			for(var i=0;i<projectAreas.length;i++) {
				if (projectAreas[i].id != this._currentProjectArea.itemId) {
					if (!this.projectAreaItemId || this.projectAreaItemId == projectAreas[i].id) {
						EditorUtil._addOption(this._projectSelect, new Option(projectAreas[i].label, projectAreas[i].id));
					}	
				}
			}
			
			if(this._projectSelect.childNodes.length === 0) this._projectSelect.style.visibility = "hidden";
			else if(this._projectSelect.childNodes.length === 1) this._projectSelect.disabled = true;
			this._isProjectAreaInitialized = true;
			this._resetTypeFilterSelector();
		},
		_collectProcessAreas: function(processAreas, collector, indent, includeChildren, disable){
			if (processAreas) {
				for (var i = 0; i < processAreas.length; i++) {
					var team = processAreas[i];
					var item = {
						id: team.itemId,
						label: indent + team.name,
						disabled: disable
					};
					collector.push(item);		
					if (includeChildren) 
						this._collectProcessAreas(team.children, collector, indent + "   ", true, false);
				}
			}
		},
		_resetTypeFilterSelector: function(doSearchAfter) {
			this._searchPending = doSearchAfter;
			if(this._typeSelect)	this.currentTypeSelection = this._typeSelect.value;
			this._typeSelect = document.createElement("select");
			this._typeSelect.className = "selectSelect";
			this.connect(this._typeSelect, "onchange", "_doSearch");
			dojox.data.dom.replaceChildren(this.typeSelectDiv,  this._typeSelect);
			if(!this._isProjectAreaInitialized) return; // will call back after init
			if(this._projectSelect.childNodes.length === 0) return; // no types to get
			var workItemProxy = this._getWorkingCopy(); 
			var projectAreaItemId = this._projectSelect.value;
			workItemProxy.fetchWorkItemTypes({onSuccess: dojo.hitch(this, this._resetTypeFilterSelector2), projectAreaItemId: projectAreaItemId })
		},
		_resetTypeFilterSelector2: function(workItemTypes) {
			EditorUtil._addOption(this._typeSelect, new Option(this.messages.optionShowAll, -1));
			if(!workItemTypes || workItemTypes.length == 0) return;
			for(var i=0;i<workItemTypes.length;i++) {
				if(!this._isTypeIdShown(workItemTypes[i].id)) continue;
				EditorUtil._addOption(this._typeSelect, new Option(workItemTypes[i].label, workItemTypes[i].id), /* isDefault */ this.currentTypeSelection ==  workItemTypes[i].id);
			}
			
			if(this._projectSelect.childNodes.length === 0) this._projectSelect.style.visibility = "hidden";
			else if(this._projectSelect.childNodes.length === 1) this._projectSelect.disabled = true;
			
			if(this._searchPending) {
				this._searchPending = false;
				this._doSearch();
			}
		},
		_isTypeIdShown: function(workItemType) {
			if(!workItemType || !this.workItemTypeIdList || this.workItemTypeIdList.length == 0) return true;
			for(var i=0;i<this.workItemTypeIdList.length; i++) {
				if(workItemType == this.workItemTypeIdList[i]) return true;
			}
			return false
		},
		setFocus: function() {
			if(this.showRecentlyViewed) {
				this._showRecentlyViewedList();
			}	
			this.queryInput.focus();
		},
		reset: function() {
			this.queryInput.value= "";
			this.queryResults.length= 0;
			this._clearStatusMessage();
			this.okCmd.disabled= true;
		},
		setKey: function(key) {
			this.key= key;
		},
		
		setAllowsMultiple: function(allowsMultiple) {
			this.allowsMultiple= allowsMultiple;
		},
		
		setExclusionList: function(exclusionList) {
			this.exclusionMap= {};
			for (var i=0; i<exclusionList.length; i++) {
				var id= exclusionList[i];
				this.exclusionMap[id]= true;
			}
		},
		
		_connectEvents: function() {
			this.connect(this.okCmd, "onclick", "_handleOk");
			this.connect(this.queryResults, "ondblclick",  "_handleOk");
			this.connect(this.cancelCmd, "onclick",  "_handleCancel");
			this.connect(this.queryInput, "onkeyup", "_doCancellableSearch");
			this.connect(this.queryResults, "onkeyup",  "_handleQueryResultsKey");
			this.connect(this.queryResults, "onchange", "_handleSelectionChange");
			this.connect(this._projectSelect, "onchange", "_handleProjectChange");
		},
		_handleProjectChange: function(e){
			this._resetTypeFilterSelector(true /* do search */); // will force a search
		},
		_handleQueryResultsKey: function(e) {
			if(e.keyCode === KEY_ENTER || e.keyCode === KEY_RETURN || e.keyCode === KEY_SPACE) {// enter
				this._handleOk();
			}
		},
		
		_handleOk: function() {
			var retArr= [];
			var options= this.queryResults.options;
			for (var i=0; i<options.length; i++) {
				if (options[i].selected) {
					retArr.push(this._backingWorkItems[options[i].value]);
				}
			}
			this.onOk(retArr, this.key);
			this.reset();			
		},
		
		_handleCancel: function() {
			this.onCancel();
			this.reset();
		},
		
		_handleSelectionChange: function() {
			// disable the 'OK' button if nothing is selected
			this.okCmd.disabled= this.queryResults.selectedIndex === -1;
			if (!this.allowsMultiple) {
				this._deselectPreviouslySelected();
			}
		},
		
		_deselectPreviouslySelected: function() {
			var options= this.queryResults.options;
			var prevOption= this._currentOption ? this._currentOption : null;
			var newOptionSet= false;

			for (var i=0; i<options.length; i++) {
				if (!options[i].selected) {
					continue;
				} else if (options[i] == prevOption || newOptionSet) {
					options[i].selected= false;
				} else {
					this._currentOption= options[i];
					newOptionSet= true;
				}
			}
		},
		
		_doCancellableSearch: function(e) {
			if(e.keyCode === 40){ // arrow down 
				this.queryResults.focus();
				return;
			} 
			if(e.keyCode === 38){ // arrow up 
				this._projectSelect.focus();
				return;
			} 
			if (this._currentId) {
				clearTimeout(this._currentId);
			}
			var self= this;
			var doSearchFunc= function() {
				self._doSearch();
			};
			this._currentId= setTimeout(doSearchFunc, DELAY);
		},
		
		_doSearch: function() {
			// if there's no input, do nothing
			var value= this.queryInput.value;
			if (dojo.trim(value) === "") {
				this.queryResults.length= 0;
				this._clearStatusMessage();
				return;
			}
			var srh= new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");
			var params;
			var projectAreaItemId = this._projectSelect.value;
			if (WorkItemUtil.isWorkItemId(value))
				params= { id: WorkItemUtil.getWorkItemId(value), projectAreaItemId: projectAreaItemId, scopeToProject: false };
			else
				params= { fullText: value, projectAreaItemId : projectAreaItemId };
			var workItemType = this._typeSelect.value;
			if(workItemType && workItemType != -1) {
				params.workItemType = workItemType;
			}
			this._setStatusMessage(this.messages.busyRunningSearch);
			QueryClient.getResults(params, srh);
		},
		/*
		 * Get the recent items that are workitems
		 *    id 
		 *    summary
		 */
		_getRecentWorkItems: function() {
			if(this.parentController && this.parentController.pageController)
					return this.parentController.pageController.getRecentWorkItemList(); // TODO: Move recent list to cache
			return [];
		},
		_getWorkingCopy: function() {
			if(this.workingCopy) return this.workingCopy;
			if(this.parentController && this.parentController.workingCopy) return this.parentController.workingCopy;
			this.workingCopy = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({ id: -1, createIfNeeded: true, incrementUseCount: false, doNotCache: true});
			return this.workingCopy;
		},
		_getCurrentWorkItem: function() {
			var currentWorkItem;
			if(this.parentController && this.parentController.workingCopy) return this.parentController.workingCopy.object;		
			if(this.parentController && this.parentController.pageController) {
				currentWorkItem = this.parentController.pageController.getCurrentWorkItem();
			} else {
				if (this.parentController) {
					if(this.parentController.workItem) {
						currentWorkItem = this.parentController.workItem;
					} else {
						if(this.parentController._workItem) {
							currentWorkItem = this.parentController._workItem;
						}
					}
				}
			}
			return currentWorkItem;
		},
		/*
		 * Return the current work item id, using lazy initialization
		 */
		_getCurrentWorkItemId: function() {
			if(this._currentWorkItemId) return this._currentWorkItemId;
			var currentWorkItem = this._getCurrentWorkItem();
			
			var currentWorkItemId = -1;
			if(currentWorkItem) {
				currentWorkItemId = currentWorkItem.id;
			}
			this._currentWorkItemId = currentWorkItemId;
			return this._currentWorkItemId;
		},
		/*
		 * Show the recently viewed list
		 */
		_showRecentlyViewedList: function() {
			var currentWorkItemId = this._getCurrentWorkItemId();
			try{ 
				var currentWorkItem = this._getCurrentWorkItem();
				if(currentWorkItemId > 0 && currentWorkItem && currentWorkItem.attributes && currentWorkItem.attributes.workItemType && currentWorkItem.attributes.workItemType.label) {
					var currentLabel = dojo.string.substitute(this.messages.labelRecentlyViewedExcluding , [currentWorkItem.attributes.workItemType.label, "" + currentWorkItemId ]);
				} else {
					var currentLabel = this.messages.labelRecentlyViewedExcludingThisWorkItem;					
				}
				var title = currentLabel;
//				var title = dojo.string.substitute(this.messages.labelRecentlyViewedExcluding , [currentLabel]);
				dojox.data.dom.replaceChildren(this.resultTitle, document.createTextNode(title));
			} catch(ignore) {
				this.resultTitle.innerHTML = this.messages.labelRecentlyViewed; /* innerHTML OK */
			};
			this._backingWorkItems= {};
			var recentItems = this._getRecentWorkItems();
			for (var i=0; i<recentItems.length; i++) {
				if(currentWorkItemId == recentItems[i].id || recentItems[i].id < 0) {
					continue; // don't put the current item on the list; don't put new items on the list
				}
				/*
				 * Filter to show only allowed types
				 */
				if(recentItems[i] && recentItems[i].attributes && recentItems[i].attributes.workItemType) {
					if(!this._isTypeIdShown(recentItems[i].attributes.workItemType.id)) continue;	
				}

				var title= EditorUtil.getSummaryLabel(recentItems[i], /* skip type */ true);
				var option= new Option(title, recentItems[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[recentItems[i].id]= recentItems[i];
				if (this.exclusionMap && this.exclusionMap[recentItems[i].id]) {
					this._disableOption(option,recentItems[i].id );
				}
			}
		},
		_loadQueryResults: function(summaryPage) {
			this.resultTitle.innerHTML = this.messages.labelMatching; /* innerHTML OK */
			var summaries= summaryPage.workItemSummaryDTOs ?
					summaryPage.workItemSummaryDTOs : [];
			this.queryResults.length= 0;
			this._backingWorkItems= {};
			var displayedResults= 0;
			var typeId = this._typeSelect ? this._typeSelect.value : -1;
			for (var i=0; i<summaries.length; i++) {
				if(typeId != -1) {
					if(typeId != summaries[i].type2 ) continue;
				} else {
					/*
					 * Filter the results to the selected types
					 */
					 if(!this._isTypeIdShown(summaries[i].type2)) continue;
				}
				displayedResults++;
				var title= summaries[i].id + ": " + EditorUtil.restoreTags(summaries[i].summary);
				var option= new Option(title, summaries[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[summaries[i].id]= summaries[i];
				if (this.exclusionMap[summaries[i].id]) {
					this._disableOption(option,summaries[i].id);
				}	
			}
			this._setStatusMessage(dojo.string.substitute(this.messages.msgResults, [ displayedResults ]));
		},
		_disableOption: function(option, id){
			var currentWorkItemId = this._getCurrentWorkItemId();
			option.style.color = "#CCCCCC";
			option.selected = false;
			if(id == currentWorkItemId) {
				option.title = this.messages.titleThisIsCurrent;
			} else {
				option.title = this.messages.titleThisIsLinked;
			}
			
		},
		_queryError: function(errorObj) {
			this._setStatusMessage(this.messages.errorFetchingSearchResults, /*isError*/ true);
		},
		
		_clearStatusMessage: function() {
			this.msgArea.innerHTML= ""; /* innerHTML OK */
		},
		
		_setStatusMessage: function(msg, isError) {
			dojox.data.dom.replaceChildren(this.msgArea, document.createTextNode(msg));
			this.msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
		}
	}
);

})();