/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ListAttributePart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItem");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var EventTopics				= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ListAttributePart",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="EnumerationWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" />' +
						'<div class="fieldWrapper">' +
						'<select dojoAttachPoint="_select" dojoAttachEvent="onchange: _handleSelectChange" multiple="true" class="Select">'  +
						'</select>' +
						'</div>' +
						'</div>',

		postCreate: function() {
			WidgetUtil.setSpacerImage(this._image);

			this._modifiers = this._modifiers ? this._modifiers : {};
			this.value = this.values[0].content;

			// If we have an associated enumeration valueset, attempt to
			// get it and fill in options later.
			this._fetchEnumeration({'enumerationId': this._getBoundEnumeration()});
				
			if (this._modifiers && this._modifiers.onCreate) {
				this.parentWidget[this._modifiers.onCreate](this, this.value);
			}
			if (this._modifiers && this._modifiers.onValueChanged) {
				this.parentWidget[this._modifiers.onValueChanged](this , this.value, /* isInitialization */ true);
			}
			if (this.workingCopy.isGuest()) {
				dojo.addClass(this._select, "GuestField");
			}
			this.isChanged = false;
			this.__workItemAdapter= new com.ibm.team.workitem.web.client.internal.WorkItem(this.workingCopy);
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: EventTopics.ON_DEPENDENCY_CHANGE,
				listener: this,
				functionName: "_fetchValueSet"
			});
		},
		_handleAttributeChange: function(parms) {
			this.setValue(parms.attributeValue);
			this._bindValuesToSelect(this._possibleValues, this._select, this._image);
		},
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.attributeId] });			
		},
		_setAttributeValue: function(value, isInit){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "content"],
				savedAttributePath: ["attributes", this.attributeId, "content"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				value: value
			});
		},		
		_getBoundEnumeration: function() {
			var props = this.presentation.properties;
			var enumName = null;
			
			for (var i = 0; props && i < props.length; i++) {
				if (props[i].key == "enumeration") {
					enumName = props[i].value;
					break;
				}
			}
			return enumName;
		},

		_createOption: function(select, enumLit) {
			if (dojo.isIE) {
				return new Option((enumLit.label) ? enumLit.label : enumLit.id, enumLit.id);
			}
			var opt = new Option("", enumLit.id);
			var span = dojo.doc.createElement("span");
			if (enumLit.iconUrl) {
				var img = dojo.doc.createElement("img");
				img.src=enumLit.iconUrl;
				img.style.verticalAlign = "middle";
				opt.appendChild(img);
			}
			span.innerHTML = (enumLit.label) ? enumLit.label : enumLit.id;
			span.style.verticalAlign = "middle";
			opt.appendChild(span);
			return opt;	
		},
		
		_bindValuesToSelect: function(possibleValues, select, image) {
			dojox.data.dom.removeChildren(select);
		    var opts = select.options;
			dojo.forEach(possibleValues, function(enumLit) {
				opts[opts.length] = this._createOption(select, enumLit);
			}, this);

			var vals = this._splitValues(this.value);
			dojo.forEach(vals, function(val) {
			    var found = false; 
				dojo.forEach(select.options, function(option) {
					if (option.value === val) {
						option.selected = true;
						found = true;
					}
				});
				// If the val wasn't part of the options, add it.
				if (found === false) {
					var opt = this._createOption(select, {id : val});
					EditorUtil._addOption(select, opt);
					opt.selected = true;
				}
			}, this);
		},

		_handleSelectChange: function(e) {
			this.isChanged=true;
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}
			var select= EditorUtil._getOriginalTarget(e);
			var selectValue = this._joinValues(this._getCurrentSelectProps(select));
			
			if (select === this._select) {
				if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
					this.parentWidget[this._modifiers.onValueChanged](this , selectValue, /* isInitialization */ false);
				}
			}
			this._setAttributeValue(this.getValue());
		},
		 setValue: function(value) {
		 	// Assume that value is a single string with comma-separated values.
		 	var newValue = value;
			// TODO: may need to validate values
			this.value = newValue;
		 },
	     getValue: function() {
			var val = this._joinValues(this._getCurrentSelectProps(this._select));
			return val;
		 },
		 _getCurrentSelectProps: function(select) {
			var options = select.options;
			var props = new Array();
			for (var i = 0; i < options.length; i++) {
				var option = options[i];
				if (option.selected) {
					var prop = {
						option: option,
						label: option.title,
						id: option.value};
					// HACK: for now, just return the value
					props[props.length] = option.value;
				}
			}
			return props;
		},
		setReadOnly: function(isReadOnly) {
			this._select.disabled=isReadOnly;
		},		
		_splitValues: function(vals) {
			if (dojo.isString(vals)) {
				if (vals === "") {
					return new Array();
				} else {
					return vals.split(',');
				}
			}
			return null;
		},
		_joinValues: function(arr) {
			if (dojo.isArray(arr)) {
				return arr.join(',');
			}
			return "";
		},
		_fetchEnumeration: function(params) {
			if (params.enumerationId) {
				var srh= this._createValueSetHandler();
				var args = {
					enumerationId: params.enumerationId,
					projectAreaItemId: this.workingCopy.getProjectAreaItemId()
				}
				WorkItemClient.getEnumerationValues(srh, args);
			} else {
				this._possibleValues = new Array();
				this._bindValuesToSelect(this._possibleValues, this._select, this._image);
			}
		},
		
		_fetchValueSet: function() {
			var srh= this._createValueSetHandler();
			this.__workItemAdapter.getValueSet(this.attributeId, srh);
		},
		
		_createValueSetHandler: function() {
			var self= this;
			var serviceObj= {
				success: function(values) {
					if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
					self._possibleValues = values;
					self._bindValuesToSelect(values, self._select, self._image);
					//self.updateValidationMessage( !self._isAutomaticallySettingResolutionToDuplicate);
				},
				failure: function(errorObj) {
					if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
					self.parentController.handleError(errorObj, "A problem occurred while trying to fetch enumeration.");
				}
			};
			return new ServiceResponseHandler(serviceObj, "success", "failure");
		},
		
		__sentinel: null
	}
);

})();
