/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection");

dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var SectionContainer= com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer;
var AttributeChooser= com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser;
var ConditionSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection;
var Term= com.ibm.team.workitem.web.client.internal.query.Term;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/TermSection.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.parent= args.parent;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this._menuItems = [{ operator: Term.AND, label: this.messages.labelAllMustMatch}, { operator: Term.OR, label: this.messages.labelAnyCanMatch}];
				
		this.attributes= args.attributes;
		this.operator= args.operator || Term.AND;
		this.term= args.term;
			
		this._conditionSections= new Array();
		this._termSections= new Array();
		this._childWidgets = new Array();
	},
	
	postCreate: function() {
		if (this.term)
			this._setTerm(this.term);
		var attributeChooser= new AttributeChooser({ termSection: this, attributeTree: this.attributes });
		this._attributeChooserNode.appendChild(attributeChooser.domNode);
		this._childAttributeChooser = attributeChooser;
	},
	
	destroy: function() {
		try {
			if(this._childAttributeChooser) this._childAttributeChooser.destroy();
			dojo.forEach(this._termSections, function(section) {
				if(section.destroy) section.destroy();
			});
			dojo.forEach(this._conditionSections, function(section) {
				if(section.destroy) section.destroy();
			});
			dojo.forEach(this._childWidgets, function(widget) {
				if(widget.destroy) widget.destroy();
			})					
			this.inherited(arguments);	
		} catch (e) {
			
		}		
	},
	
	setSectionSite: function(site) {
		this._site= site;
		site.setTitle(this._getTitle());
		site.setupMenu("label", dojo.hitch(this, this._setOperator));
		site.setMenuItems(this._menuItems, dojo.hitch(this, this._setOperator));
		site.setSelectedMenuItem(this._menuItems[this.operator == Term.AND ? 0 : 1]);
	},
	
	deregister: function(section) {
		this._remove(this._conditionSections, section);
		this._remove(this._termSections, section);
		if(section && section.destroy) section.destroy();
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	addSection: function(/* String */ attributeId, /* AttributeExpressionDTO */ expression) {
		
		var section= new ConditionSection({ attributeId: attributeId, parent: this, expression: expression });
		var container= new SectionContainer({ sectionWidget: section });
		dojo.addClass(container.domNode, "conditionLayout");
		
		this._childWidgets.push(container);
		this._conditionSections.push(section);
		
		var conditionsNode= this._conditionsNode;
		var attributeChooserNode= this._attributeChooserNode;
		conditionsNode.insertBefore(container.domNode, attributeChooserNode);
		
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
		
		return section;
	},
	
	addTermSection: function(args) {

		var term= args.term; /* TermDTO */
		var operator= term ? term.operator : args.operator;
		
		var section= new TermSection({ attributes: this.attributes, operator: operator, term: term, parent: this });
		var container= new SectionContainer({ sectionWidget: section });
		dojo.addClass(container.domNode, "termLayout");
		
		this._termSections.push(section);
		this._childWidgets.push(container);
		var conditionsNode= this._conditionsNode;
		conditionsNode.appendChild(container.domNode);
		
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
		
		if (section._childAttributeChooser) {
			setTimeout(function() {section._childAttributeChooser._addConditionAnchor.focus();}, 100);
		}
		
		return section;
	},
	
	/* Term */ getTerm: function() {
		var conditionSections= this._conditionSections;
		var termSections= this._termSections;
		
		var term= new Term(this.operator);
		for (var i= 0; i < conditionSections.length; i++) {
			var expression= conditionSections[i].getExpression();
			if (expression)
				term.addAttributeExpression(expression);
		}
		for (var k= 0; k < termSections.length; k++) {
			var subTerm= termSections[k].getTerm();
			term.addTerm(subTerm);
		}
		return term;
	},
	
	getOperator: function() {
		return this.operator;
	},
	
	onClose: function() {
		if (this.parent) {
			this.parent.deregister(this);
			return true;
		}
		return false;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	_setTerm: function(/* TermDTO */ term) {
		
		this._setOperator( { operator: term.operator });
		
		var attributeExpressions= term.attributeExpressions;
		var termExpressions= term.termExpressions;
		
		if (attributeExpressions) {
			for (var i= 0; i < attributeExpressions.length; i++) {
				var expression= attributeExpressions[i];
				this.addSection(expression.attribute.id, expression);
			}
		}
		if (termExpressions) {
			for (var i= 0; i < termExpressions.length; i++) {
				var termExpression= termExpressions[i];
				this.addTermSection({
					term: termExpression
				});
			}
		}
	},
		
	_remove: function(arr, obj) {
		var idx= dojo.indexOf(arr, obj);
		if (idx > -1)
			arr.splice(idx, 1);
	},
	
	_setOperator: function(obj) {
		this.operator= obj.operator;
		if (this._site)
			this._site.setTitle(this._getTitle());
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_getTitle: function() {
		return this.operator == Term.AND ? this.messages.labelAND : this.messages.labelOR;
	}
	
});

var TermSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection;

})();
