/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {

var Layouts = {
	TOP_LEFT: "topLeft",
	TOP_RIGHT: "topRight",
	BOTTOM_LEFT: "bottomLeft",
	BOTTOM_RIGHT: "bottomRight"
};

dojo.declare("com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon", [dijit._Widget, dijit._Templated], {
	// summary:
	//		A simple widget that shows a 16x16 icon with an overlay.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		This is a very basic widget that accepts two urls and layout information.
	//		The first url points at a 16x16 icon to display and the second at a 7x8
	//		overlay. The layout information specifies where the overlay should appear
	//		relative to the icon.
	// icon:
	//		A string that gives a url to the desired 16x16 icon.
	// overlay:
	//		A string that gives a url to the desired overlay.
	// layout:
	//		This can be one of the string constants defined by the widget:
	//			OverlayIcon.TOP_LEFT
	//			OverlayIcon.TOP_RIGHT
	//			OverlayIcon.BOTTOM_LEFT
	//			OverlayIcon.BOTTOM_RIGHT
	//		For more advanced positioning, an object can be used with 2 integer properties.
	//		It must have either "top" or "bottom" and either "left" or "right". Each property
	//		should be a number indicating the pixels of padding from the indicated border of
	//		the icon (e.g. {left: 2, bottom: 0} would mean 2 pixels from the left side and
	//		snug against the bottom border.
	//
	// examples:
	//		new OverlayIcon({
	//			icon: myIconUrl,
	//			overlay: myOverlayUrl,
	//			layout: OverlayIcon.TOP_LEFT
	//		});
	//		new OverlayIcon({
	//			icon: myIconUrl,
	//			overlay: myOverlayUrl,
	//			layout: {top: 0, right: 2}
	//		});
	//		
	// since:
	//		0.7
	
	icon: "",
	overlay: "",
	layout: Layouts.BOTTOM_RIGHT,
	
	// INTERNAL PROPERTIES AND FUNCTIONS
		
	templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/widgets/templates/OverlayIcon.html"),
	
	postCreate: function() {
		if (typeof this.layout === "string")
			dojo.addClass(this._overlay, this.layout);
		else if (typeof this.layout === "object") {
			if (typeof this.layout.left === "number")
				this._overlay.style.left = this.layout.left + "px";
			else if (typeof this.layout.right === "number")
				this._overlay.style.right = this.layout.right + "px";
			if (typeof this.layout.top === "number")
				this._overlay.style.top = this.layout.top + "px";
			else if (typeof this.layout.bottom === "number")
				this._overlay.style.bottom = this.layout.bottom + "px";
		}
	}

});
dojo.mixin(com.ibm.team.scm.web.ui.internal.widgets.OverlayIcon,Layouts);
	
})();
