/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

 // Use the jslint 'good things' default with the exception of: nomen: false (don't check variable names since dijit api uses _ which is against jslint rules) and forin
 /*jslint white: true, onevar: true, browser: true, undef: true, nomen: false, eqeqeq: true, plusplus: true, bitwise: true, regexp: true, strict: true, newcap: true, immed: true */
 /*global com, dijit, dojo, dojox, net, window */
 "use strict";
 
dojo.provide("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourcesEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("dijit.form.Button");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");
dojo.require("dojo._base.Deferred");

dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("dojox.string.BidiComplex");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.UploadItem");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.NewFolderItem");
dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function () {
	var ScmUtil, ScmClient, PlatformUI, ServiceResponseHandler, SelectWorkItemDialog, checkNotNull;
	ScmUtil	= com.ibm.team.scm.web.client.util;
	ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
	PlatformUI = net.jazz.ajax.ui.PlatformUI;
	ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
	SelectWorkItemDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
	checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
	
	/**
	 * Widget to edit files directly on a stream. User can also set a comment describing the reason for the new content.
	 * 
	 * Constructor supports the following properties.
	 * 1. mode: (required)
	 *  SINGLE_FILE_UPLOAD: user can upload new content to an existing file
	 *  SINGLE_FILE_ADD: user can upload new content for a new file
	 * 	SINGLE_FILE_DELETE user can delete a single file and enter a comment.
	 *  SINGLE_FOLDER_DELETE user can delete a single folder and enter a comment.
	 *  SINGLE_FOLDER_ADD user can enter a folder name and comment.
	 *  
	 * 2. hint: (optional)
	 *  TEXT_EDIT: user can edit text content in place. Only supported by SINGLE_FILE_UPLOAD
	 *  and SINGLE_FILE_ADD mode 
	 *  
	 * 3. workspaceId, componentId, path: (required if versionableDTO is not set)
	 *  identify the resource to edit. Resource will be fetched and locked if necessary.
	 *  
	 * 4. versionableDTO: (optional) the currently known state of the resource to edit. The latest state will be
	 *  fetched. Providing the state the user is currently looking at gives a way to warn the user if a more recent
	 *  state exists in the stream aka the user needs to check the latest.
	 *  
	 * 5. pageController: (optional but strongly recommended for proper event notification such as locking update, stream update)
	 *  the scm page controller
	 *  
	 * Constructor can be passed three callbacks.
	 * 
	 * 1. onResourceEditorSaved: (optional)
	 * 	invoked when save completed successfully (optional)
	 * 2. onResourceEditorCancelled: (optional) 
	 *  invoked when user hits cancel
	 * 3. onResourceEditorClose: (optional)
	 *  invoked when either saved completed or user hits cancel
	 */
	dojo.declare(
			"com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.ResourcesEditor", 
			[dijit._Widget, dijit._Templated, dijit._Container], {
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/widgets/resourcesEditor/templates/ResourcesEditor.html"),
		widgetsInTemplate : true,
		constructor: function (params) {
			this.params = params;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			if (this.params.mode === "SINGLE_FILE_DELETE") {
				this.labelTitle = this.messages.ResourcesEditor_titleDeleteSingleFile;				
			}
			if (this.params.mode === "SINGLE_FOLDER_DELETE") {
				this.labelTitle = this.messages.ResourcesEditor_titleDeleteSingleFolder;
			}
			if (this.params.mode === "SINGLE_FOLDER_ADD") {
				this.labelTitle = this.messages.ResourcesEditor_titleAddSingleFolder;
			}
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				if (this.params.hint === "TEXT_EDIT") {
					this.labelTitle = this.messages.ResourcesEditor_titleEditTextFile;					
				} else {
					this.labelTitle = this.messages.ResourcesEditor_titleUploadSingleFile;
				}
			}
			if (this.params.mode === "SINGLE_FILE_ADD") {
				if (this.params.hint === "TEXT_EDIT") {
					this.labelTitle = this.messages.ResourcesEditor_titleNewTextFile;					
				} else {
					this.labelTitle = this.messages.ResourcesEditor_titleUploadSingleFileAdd;					
				}
			}
			this.versionableDTO = params.versionableDTO;
			if (this.versionableDTO) {
				this.workspaceId = this.versionableDTO.workspace.itemId;
				this.componentId = this.versionableDTO.component.itemId;
				this.path = this.versionableDTO.path;
			} else {
				this.workspaceId = params.workspaceId;
				this.componentId = params.componentId;
				this.path = params.path;
			}
			this.labelAddComment = this.messages.ResourcesEditor_labelAddComment;
			this.labelSave = this.messages.ResourcesEditor_labelSave;
			this.labelCancel = this.messages.ResourcesEditor_labelCancel;
			this.labelNewFolder = this.messages.ResourcesEditor_progressNewFolder;
			this.labelDeleteResource = this.messages.ResourcesEditor_progressDeleteResource;
			this.labelUpdateResource = this.messages.ResourcesEditor_progressUpdatingResource;
			this.labelUploadNotComplete = this.messages.ResourcesEditor_notUploaded;
		},
		/**
		 * Describes current lock state
		 */
		getLockStatusHtmlNode: function () {
			var result, lockedBy;
			result = document.createElement("div");
			dojo.addClass(result, "com-ibm-team-scm-decorators");
			if (!this.versionableDTO || !this.versionableDTO.versionablePlusDTO) {
				result.appendChild(document.createTextNode(this.messages.UnknownLockDecorator));
				return result;
			}
			lockedBy = this.versionableDTO.versionablePlusDTO.lockedBy;
			if (!lockedBy) {
				result.appendChild(document.createTextNode(this.messages.UnlockedDecorator));
				return result;						
			}
			if (lockedBy.itemId === ScmUtil.getLoggedInContributor().itemId) {
				result.appendChild(document.createTextNode(this.messages.LockedByMeDecorator));						
				dojo.addClass(result, "com-ibm-team-scm-resourcesEditor-lockedByMe");
			} else {
				result.appendChild(document.createTextNode(dojo.string.substitute(this.messages.LockedByOtherDecorator, [lockedBy.name])));
				dojo.addClass(result, "com-ibm-team-scm-resourcesEditor-lockedByOther");		
			}
			return result;
		},
		postCreate: function () {
			var child, d;
			
			// Add appropriate child widgets
			if (ScmUtil.workItemsInstalled()) {
				dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.WorkItemLinksItem");
				child = document.createElement("div");
				this._workitemLinksSectionDiv.appendChild(child);
				this.workItemLinksItemWidget = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.WorkItemLinksItem(
				{
				},
				child
				);
				this.workItemLinksItemWidget.startup();
			}			
						
			// Start running the preconditions (fetch latest versionable, lock if necessary)
			// Note. the following child widgets could be modified to be visible even before the preconditions
			// were executed (for better user feedback)
			// Sequence:
			// 1. If resource is unlocked: fetch -> lock -> refetch -> show child widget
			// 2. If resource is locked: fetch -> show child widget
			if (this.params.mode === "SINGLE_FILE_UPLOAD" || this.params.mode === "SINGLE_FILE_DELETE" || this.params.mode === "SINGLE_FOLDER_DELETE") {
				d = this.fetchVersionable();
				d.addCallback(dojo.hitch(this, function () {
					var lockedBy, lockedByOther, size, contentType, encoding;
					
					if (this.params.mode === "SINGLE_FILE_UPLOAD" && this.params.hint === "TEXT_EDIT") {
						// Verify resource is text (so it doesn't get corrupted in text edit)
						contentType = ScmUtil.getVersionableDTOPropertyValue(this.versionableDTO, "contentType");
						if (!contentType || !ScmUtil.isKnownTextContentType(contentType)) {
							if (!confirm(dojo.string.substitute(this.messages.ResourcesEditor_PromptNotText, [ contentType ? contentType : this.messages.ResourcesEditor_UnknownContentType]))) {
								// Close editor
								this.onClose();
								return;
							}
						}
						// Verify resource is less than 1MB (so UI doesn't freeze)
						size = this.versionableDTO.versionable.content.size;
						if (size > 1024 * 1024) {
							if (!confirm(dojo.string.substitute(this.messages.ResourcesEditor_PromptTooBig, [ ScmUtil.getSizeDisplay(size, true) ]))) {
								// Close editor
								this.onClose();
								return;								
							}
						}
						// Verify resource's encoding is UTF-8
						encoding = this.versionableDTO.versionable.content.characterEncoding;
						if (encoding !== "UTF-8") {
							if (!confirm(dojo.string.substitute(this.messages.ResourcesEditor_PromptNotUTF8, [ encoding ]))) {
								// Close editor
								this.onClose();
								return;								
							}							
						}
					}
										
					lockedBy = this.versionableDTO && this.versionableDTO.versionablePlusDTO && this.versionableDTO.versionablePlusDTO.lockedBy;
					lockedByOther = lockedBy && lockedBy.itemId !== ScmUtil.getLoggedInContributor().itemId;
										
					// If resource is unlocked, ask user to confirm auto lock
					// This is important if user downloaded / edited an old version and want to upload it - it may not be the latest anymore
					// Only a pb for mod case, non text edit - in text edit case, user locks then get the latest locked content
					if (this.params.mode === "SINGLE_FILE_UPLOAD" && this.params.hint !== "TEXT_EDIT") {
						if (!lockedBy) {
							if (!confirm(this.messages.ResourcesEditor_PromptUnlocked)) {
								// Close editor
								this.onClose();
								return;
							}
						}
						if (lockedByOther) {
							alert(dojo.string.substitute(this.messages.ResourcesEditor_PromptLockedByOther, [ lockedBy.name ]));
							// Close editor
							this.onClose();
							return;
						}
					}
					
					// If resource is unlocked, lock and refetch the new state
					if (!lockedBy) {
						// go ahead: auto lock
						d.addCallback(dojo.hitch(this, this.lock));
						d.addCallback(dojo.hitch(this, this.fetchVersionable));
						d.addCallback(dojo.hitch(this, function () {
							this.editorLocked = true;			
						}));
					}
					// add both - even if lock fail, we show child widget and let user retry grabbing lock
					d.addBoth(dojo.hitch(this, function () {
						if (this.params.mode === "SINGLE_FILE_UPLOAD") {
							child = document.createElement("div");
							this._itemsSectionDiv.appendChild(child);
							this._createUploadWidget(child);
						}
					}));
				}));
			}
			if (this.params.mode === "SINGLE_FOLDER_ADD" || this.params.mode === "SINGLE_FILE_ADD") {
				this.fetchVersionable()
				.addCallback(dojo.hitch(this, function () {
					if (this.params.mode === "SINGLE_FILE_ADD") {
						child = document.createElement("div");
						this._itemsSectionDiv.appendChild(child);
						this.uploadItemWidget = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.UploadItem(
						{
							mode : this.params.mode,
							hint : this.params.hint
						},
						child
						);
						this.uploadItemWidget.startup();
					}
					if (this.params.mode === "SINGLE_FOLDER_ADD") {
						child = document.createElement("div");
						this._itemsSectionDiv.appendChild(child);
						this.newFolderItemWidget = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.NewFolderItem(
						{
						},
						child
						);
						this.newFolderItemWidget.startup();						
					}
				}));
			}
		},
		_onResourceEditorSaved: function (newPath) {
			if (this.params.onResourceEditorSaved) {
				this.params.onResourceEditorSaved(newPath);
			}
			this.dispose();
		},
		_onResourceEditorCancelled: function () {
			if (this.params.onResourceEditorCancelled) {
				this.params.onResourceEditorCancelled();
			}
			this.dispose();
		},
		dispose: function () {
			if (this.params.onResourceEditorClose) {
				this.params.onResourceEditorClose();
			}			
			if (!this.domNode) {
				return;
			}
			this.destroy();
		},
		showWidget: function () {
		},
		_createUploadWidget: function (parentNode) {
			this.uploadItemWidget = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.UploadItem(
			{
				mode : this.params.mode,
				hint : this.params.hint,
				versionableDTO : this.versionableDTO
			},
			parentNode
			);
			
			this.uploadItemWidget.startup();
		},
		/**
		 * Lock the versionable managed by this editor
		 */
		lock: function () {
			var operation;
			if (this.versionableDTO.versionable["com.ibm.team.repository.typeName"] === "com.ibm.team.scm.Folder") {
				// folder - lock all
				operation = "acquire_subtree";
			} else {
				operation = "acquire";
			}
			return this.updateLock(this.versionableDTO, operation);
		},
		/**
		 * Release the versionable managed by this editor
		 */
		unlock: function () {
			var operation;
			if (this.versionableDTO.versionable["com.ibm.team.repository.typeName"] === "com.ibm.team.scm.Folder") {
				// folder - release all
				operation = "release_subtree";
			} else {
				operation = "release";
			}
			return this.updateLock(this.versionableDTO, operation);
		},
		/**
		 * Operation: "acquire" or "release"
		 */
		updateLock: function (versionableDTO, operation) {
			var typeName, itemNs, itemTypeName, d, message, that, args;
			d = new dojo.Deferred();
			that = this;
			typeName = versionableDTO.versionable["com.ibm.team.repository.typeName"];
			if (typeName === "com.ibm.team.filesystem.FileItem") {
				itemNs = "com.ibm.team.filesystem";
				itemTypeName = "FileItem";
			}
			if (typeName === "com.ibm.team.scm.Folder") {
				itemNs = "com.ibm.team.scm";
				itemTypeName = "Folder";
			}
			args = 				{
				operation : [ operation ],
				streamId : versionableDTO.workspace.itemId,
				componentId : versionableDTO.component.itemId,
				versionableNamespace : [ itemNs ],
				versionableItemType : [ itemTypeName ],
				versionableItemId : [ versionableDTO.versionable.itemId ]
			};
			message = operation === "acquire" || operation === "acquire_subtree" ? this.messages.StreamWorkspaceView_66 : this.messages.StreamWorkspaceView_67;
			PlatformUI.getWorkbench().showMessage(message);
			ScmClient.postUpdateLocks(new ServiceResponseHandler(
			{
				handler : function () {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.params.pageController) {
						that.params.pageController.onUpdateLocks(args);
					}					
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;
		},
		addFolder: function () {
			var wItemId, cId, parentId, newFolderName, vItemId, csComment, IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID, that, d, args;
			d = new dojo.Deferred();
			that = this;
			wItemId = this.versionableDTO.workspace.itemId;
			cId = this.versionableDTO.component.itemId;
			parentId = this.versionableDTO.versionable.itemId;
			newFolderName = this.newFolderItemWidget.getFolderName();
			IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID = "_xf9MgJcPEd6rDbE8csAykA";
			vItemId = IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID;
			csComment = this._textArea.value;
			args = {
				streamId : wItemId,
				componentId : cId,
				changeSetComment : csComment,
				instruction :  [ "UPDATE" ],
				unlockWhenDone : [ false ],
				versionableItemId : [ vItemId ],
				versionableItemType : [ "Folder" ],
				versionableNamespace : [ "com.ibm.team.scm" ],
				parentId : [ parentId ],
				nameInParent : [ newFolderName ]
			};
			PlatformUI.getWorkbench().showMessage(this.labelNewFolder);
			ScmClient.postUpdateStream(new ServiceResponseHandler(
			{
				handler : function (response) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					that.changeSetItemId = response;
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.params.pageController) {
						args.changeSetItemId = response;
						that.params.pageController.onUpdateStream(args);
					}					
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;			
		},
		deleteFile : function () {
			var wItemId, cId, vItemId, csComment, that, d, args;
			d = new dojo.Deferred();
			that = this;
			wItemId = this.versionableDTO.workspace.itemId;
			cId = this.versionableDTO.component.itemId;
			vItemId = this.versionableDTO.versionable.itemId;
			csComment = this._textArea.value;
			args = {
				streamId : wItemId,
				componentId : cId,
				changeSetComment : csComment,
				instruction :  [ "DELETE" ],
				versionableItemId : [ vItemId ],
				versionableItemType : [ "FileItem" ],
				unlockWhenDone : [ true ],
				versionableNamespace : [ "com.ibm.team.filesystem" ]
			};
			PlatformUI.getWorkbench().showMessage(this.labelDeleteResource);
			ScmClient.postUpdateStream(new ServiceResponseHandler(
			{
				handler : function (response) {
					var path, parentPath;
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					that.changeSetItemId = response;
					if (that.params.pageController) {
						args.changeSetItemId = response;
						that.params.pageController.onUpdateStream(args);
					}					
					PlatformUI.getWorkbench().hideMessageArea();
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;
		},		
		deleteFolder : function () {
			var wItemId, cId, vItemId, csComment, that, d, args;
			d = new dojo.Deferred();
			that = this;
			wItemId = this.versionableDTO.workspace.itemId;
			cId = this.versionableDTO.component.itemId;
			vItemId = this.versionableDTO.versionable.itemId;
			csComment = this._textArea.value;
			args = {
				streamId : wItemId,
				componentId : cId,
				changeSetComment : csComment,
				instruction :  [ "DELETE" ],
				unlockWhenDone : [ true ],
				versionableItemId : [ vItemId ],
				versionableItemType : [ "Folder" ],
				versionableNamespace : [ "com.ibm.team.scm" ]
			};
			PlatformUI.getWorkbench().showMessage(this.labelDeleteResource);
			ScmClient.postUpdateStream(new ServiceResponseHandler(
			{
				handler : function (response) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					that.changeSetItemId = response;
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.params.pageController) {
						args.changeSetItemId = response;
						that.params.pageController.onUpdateStream(args);
					}					
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;			
		},
		updateFile : function () {
			var d, wItemId, cId, vItemId, csComment, that, i, nameInParent, parentId, contentHashId, unlockWhenDone, contentType, isExecutable, delimiter, delimiterCount, encoding, contentLength, timestamp, DELIBERATELY_NULL_UUID, IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID, previousOwnerItemId, previousOwnerStateId, args;			
			that = this;
			d = new dojo.Deferred();
			this.uploadResult = this.uploadItemWidget.getUploadValues();
			wItemId = this.versionableDTO.workspace.itemId;
			cId = this.versionableDTO.component.itemId;
			DELIBERATELY_NULL_UUID = "Null_JuvZIKbREd6Hkq-ObO81hQ";
			previousOwnerItemId = DELIBERATELY_NULL_UUID;
			previousOwnerStateId = DELIBERATELY_NULL_UUID;
			isExecutable = this.uploadResult.executable;
			delimiter = this.uploadResult.lineDelimiter;
			delimiterCount = this.uploadResult.lineDelimiterCount;
			contentHashId = this.uploadResult.contentHash;
			contentLength = this.uploadResult.contentLength;
			nameInParent = this.uploadResult.filename;
			contentType = this.uploadResult.contentMimeType;
			encoding = this.uploadResult.textFileEncoding;
			timestamp = new Date().getTime();
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				vItemId = this.versionableDTO.versionable.itemId;
				parentId = this.versionableDTO.versionable.parentId;
				unlockWhenDone = true;								
			}
			if (this.params.mode === "SINGLE_FILE_ADD") {
				IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID = "_xf9MgJcPEd6rDbE8csAykA";
				vItemId = IFILESYSTEM_REST_SERVICE2_NEW_ITEM_ID;
				parentId = this.versionableDTO.versionable.itemId;
				unlockWhenDone = false;
			}
			csComment = this._textArea.value;
			args = {
				streamId : wItemId,
				componentId : cId,
				changeSetComment : csComment,
				instruction :  [ "UPDATE" ],
				unlockWhenDone : [ unlockWhenDone ],
				versionableItemId : [ vItemId ],
				versionableItemType : [ "FileItem" ],
				versionableNamespace : [ "com.ibm.team.filesystem" ],
				contentHashId : [ contentHashId ],
				contentType : [ contentType ],
				isExecutable : [ isExecutable ],
				delimiter : [ delimiter ],
				delimiterCount : [ delimiterCount ],
				encoding : [ encoding ],
				contentLength : [ contentLength ],
				timestamp : [ timestamp ],
				parentId : [ parentId ],
				nameInParent : [ nameInParent ],
				previousOwnerItemId : [ previousOwnerItemId ],
				previousOwnerStateId : [ previousOwnerStateId ]			
			};
			PlatformUI.getWorkbench().showMessage(this.labelUpdateResource);
			ScmClient.postUpdateStream(new ServiceResponseHandler(
			{
				handler : function (response) {
					var path, parentPath;
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					that.changeSetItemId = response;
					PlatformUI.getWorkbench().hideMessageArea();
					if (that.params.pageController) {
						args.changeSetItemId = response;
						that.params.pageController.onUpdateStream(args);
					}					
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
			args
			);
			return d;			
		},
		associateWorkItems : function () {
			var d, that, workItemIds;
			d = new dojo.Deferred();
			that = this;
			workItemIds = this.workItemLinksItemWidget.getWorkItemItemIds();
			if (workItemIds.length === 0) {
				d.callback(true);
				return;
			}
			PlatformUI.getWorkbench().showMessage(this.messages.ResourcesEditor_progressWorkItem);			
			ScmClient.postAddLinks(new ServiceResponseHandler(
			{
				handler : function () {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.callback(true);
				},
				error : function (error) {
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));
				}
			},
			'handler',
			'error'
			),
				{
					workspaceItemId : that.versionableDTO.workspace.itemId,
					changeSetItemId : that.changeSetItemId,
					workItemIds : workItemIds
				}
			);
			return d;			
		},		
		// callbacks for header buttons
		onRun: function () {
			this.onRunPendingFlag = true;
			this.onRefreshed();
			if (this.params.mode === "SINGLE_FILE_DELETE") {
				this.deleteFile()
				.addCallback(dojo.hitch(this, this.associateWorkItems))
				.addCallback(dojo.hitch(this, function () {
					var wItemId, cId, path, parentPath, href;
					// Check widget is still up
					if (!this.domNode) {
						return;
					}
					this._onResourceEditorSaved();
				}))
				.addBoth(dojo.hitch(this, this.clearOnRunPendingFlag));
			}			
			if (this.params.mode === "SINGLE_FOLDER_DELETE") {
				this.deleteFolder()
				.addCallback(dojo.hitch(this, this.associateWorkItems))
				.addCallback(dojo.hitch(this, function () {
					var wItemId, cId, path, parentPath, href;
					// Check widget is still up
					if (!this.domNode) {
						return;
					}
					this._onResourceEditorSaved();
				}))
				.addBoth(dojo.hitch(this, this.clearOnRunPendingFlag));
			}
			if (this.params.mode === "SINGLE_FOLDER_ADD") {
				this.addFolder()
				.addCallback(dojo.hitch(this, this.associateWorkItems))
				.addCallback(dojo.hitch(this, function () {
					// Check widget is still up
					if (!this.domNode) {
						return;
					}
					this._onResourceEditorSaved();
				}))
				.addBoth(dojo.hitch(this, this.clearOnRunPendingFlag));
			}
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				this.uploadItemWidget.checkAndRedoUpload()
				.addCallback(dojo.hitch(this, this.updateFile))
				.addCallback(dojo.hitch(this, this.associateWorkItems))
				.addCallback(dojo.hitch(this, function () {
					var wItemId, cId, path, parentPath, href;
					// Check widget is still up
					if (!this.domNode) {
						return;
					}
					
					parentPath = ScmUtil.parseParentPath(this.versionableDTO.path);
					path = parentPath + "/" + encodeURIComponent(this.uploadResult.filename);
					this._onResourceEditorSaved(path);					
				}))
				.addBoth(dojo.hitch(this, this.clearOnRunPendingFlag));				
			}
			if (this.params.mode === "SINGLE_FILE_ADD") {
				this.uploadItemWidget.checkAndRedoUpload()
				.addCallback(dojo.hitch(this, this.updateFile))
				.addCallback(dojo.hitch(this, this.associateWorkItems))
				.addCallback(dojo.hitch(this, function () {
					// Check widget is still up
					if (!this.domNode) {
						return;
					}
					this._onResourceEditorSaved();
				}))
				.addBoth(dojo.hitch(this, this.clearOnRunPendingFlag));				
			}
		},
		
		clearOnRunPendingFlag: function () {
			this.onRunPendingFlag = false;
			this.onRefreshed();
		},
		
		/**
		 * Grab the latest state of the versionable in this.versionableDTO
		 * Could warn user if state is different from the one the RE was opened against
		 * (i.e. another user delivered a change since)
		 * Require this.workspaceId, this.componentId, this.path
		 */
		fetchVersionable: function () {
			var d, that, serviceObject;
			d = new dojo.Deferred();
			checkNotNull(this.workspaceId, "workspaceId");
			checkNotNull(this.componentId, "componentId");
			checkNotNull(this.path, "path");
			
			that = this;
			
			PlatformUI.getWorkbench().showMessage(this.messages.FetchingItem);
			serviceObject = {
				success: function (versionablePlusDTO) {	
					PlatformUI.getWorkbench().hideMessageArea();
					if (versionablePlusDTO && versionablePlusDTO.versionableDTO) {
						that.versionableDTO = versionablePlusDTO.versionableDTO;
						versionablePlusDTO.versionableDTO = null;
						that.versionableDTO.versionablePlusDTO = versionablePlusDTO;
						that.onRefreshed();
						d.callback(true);
						return;
					}			
					d.errback(new Error(""));				
				},
				error: function (error) { 
					// Check widget is still up
					if (!that.domNode) {
						return;
					}
					if (error.message !== '') {
						alert(error.message);
					}
					PlatformUI.getWorkbench().hideMessageArea();
					d.errback(new Error(error.message));					
				}
			};							
						
			ScmClient.getVersionable(
				new ServiceResponseHandler(serviceObject, "success", "error"), 
					{workspace: ScmClient.WORKSPACE_PREFIX + this.workspaceId, component: ScmClient.COMPONENT_PREFIX + this.componentId}, true, this.path);
			return d;
		},			
		onStop: function () {
		},
		/**
		 * User is closing without doing any change.
		 * Release lock if editor took it and dispose widget
		 */
		onClose: function () {
			var d;
			d = new dojo.Deferred();
			d.addCallback(dojo.hitch(this, function () {
				if (this.editorLocked) {
					return this.unlock();
				}
			}));
			d.addBoth(dojo.hitch(this, function () {
				this._onResourceEditorCancelled();
			}));
			d.callback(true);
			return;
		},

		onRefreshed: function () {
			var saveEnable, showEdit;
			if (!this.domNode) {
				// too early or too late
				return;
			}
			// A new state has been reached, e.g. a new versionableDTO has been fetched
			saveEnable = true;
			if (!this.versionableDTO) {
				saveEnable = false;
			} else if (this.onRunPendingFlag) {
				saveEnable = false;				
			} else if (this.params.mode === "SINGLE_FILE_UPLOAD" || this.params.mode === "SINGLE_FILE_ADD") {
				// child widget may not be constructed yet
				saveEnable = this.uploadItemWidget && this.uploadItemWidget.isSave();
			} else if (this.params.mode === "SINGLE_FOLDER_ADD") {
				// child widget may not be constructed yet
				saveEnable = this.newFolderItemWidget && this.newFolderItemWidget.isSave();				
			}
			this._saveButton.attr("disabled", !saveEnable);
			this._saveButton.attr("label", this.onRunPendingFlag ? this.messages.ResourcesEditor_labelSaving : this.messages.ResourcesEditor_labelSave);			
		},
				
		/**
		 * Returns true if string1 ends with string2
		 */
		_stringEndsWith: function (string1, string2) {
			var index;
			index = string1.indexOf(string2);
			return index >= 0 && string2.length === string1.length - index;
		},
		destroy: function() {
			if (this.uploadItemWidget) {
				this.uploadItemWidget.destroy();
				this.uploadItemWidget = null;
			}
			if (this.newFolderItemWidget) {
				this.newFolderItemWidget.destroy();
				this.newFolderItemWidget = null;
			}
			if (this.workItemLinksItemWidget) {
				this.workItemLinksItemWidget.destroy();
				this.workItemLinksItemWidget = null;
			}
			this.inherited(arguments);
		}

	}
	);
}());