/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

 // Use the jslint 'good things' default with the exception of: nomen: false (don't check variable names since dijit api uses _ which is against jslint rules) and forin
 /*jslint white: true, onevar: true, browser: true, undef: true, nomen: false, eqeqeq: true, plusplus: true, bitwise: true, regexp: true, strict: true, newcap: true, immed: true */
 /*global com, dijit, dojo, dojox, net, window, jazz */
 "use strict";
 
dojo.provide("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.UploadItem");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("jazz.util.date");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.TextBox");

dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("dojox.string.BidiComplex");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.TextEditorItem");
dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function () {
	var ScmUtil, ScmClient, PlatformUI, ServiceResponseHandler, TeamServerClient;
	ScmUtil	= com.ibm.team.scm.web.client.util;
	TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
	ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
	PlatformUI = net.jazz.ajax.ui.PlatformUI;
	ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	/**
	 * Widget to upload file content. Supports 2 modes.
	 * SINGLE_FILE_UPLOAD : upload content for an existing file 
	 * SINGLE_FILE_ADD : add content for a new file (user can pick name)
	 * 
	 * Supports one hint (optional)
	 * TEXT_EDIT : user edits the text content to upload instead of picking up a local file
	 */
	dojo.declare(
			"com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.UploadItem", 
			[dijit._Widget, dijit._Templated, dijit._Contained, dijit._Container], {
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/widgets/resourcesEditor/templates/UploadItem.html"),
		widgetsInTemplate : true,
		constructor: function (params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			this.labelName = this.messages.UploadItem_labelName;
			this.labelExecutable = this.messages.UploadItem_labelExecutable;
			this.labelLastModifiedBy = this.messages.UploadItem_labelLastModifiedBy;
			this.labelSize = this.messages.UploadItem_labelSize;				
			this.labelContentMimeType = this.messages.UploadItem_labelContentMimeType;				
			this.labelLineDelimiter = this.messages.UploadItem_labelLineDelimiter;				
			this.labelTextFileEncoding = this.messages.UploadItem_labelTextFileEncoding;				
			this.labelAction = this.messages.UploadItem_labelAction;
			this.labelCancelUpload = this.messages.UploadItem_labelCancelUpload;
			this.labelBusyUploading = this.messages.UploadItem_labelBusyUploading;
			this.labelRemove = this.messages.UploadItem_labelRemove;
			this.labelEditFileProperties = this.messages.UploadItem_labelEditFileProperties;
			this.labelUploadFailed = this.messages.UploadItem_UploadFailed;
			this.cbLdNoneBinary = this.messages.UploadItem_cbLdNoneBinary;
			this.cbLdLFUnix = this.messages.UploadItem_cbLdLFUnix;
			this.cbLdCRMac = this.messages.UploadItem_cbLdCRMac;
			this.cbLdCRLFWindows = this.messages.UploadItem_cbLdCRLFWindows;
			this.cbPlatform = this.messages.UploadItem_cbPlatform;
			this.cbCmtAutoDetect = this.messages.UploadItem_cbCmtAutoDetect;
			this.cbCmtBinary = this.messages.UploadItem_cbCmtBinary;
			this.cbCmtPlainText = this.messages.UploadItem_cbCmtPlainText;
			this.cbCmtHtml = this.messages.UploadItem_cbCmtHtml;
			this.cbCmtXml = this.messages.UploadItem_cbCmtXml;
			this.cbCmtGif = this.messages.UploadItem_cbCmtGif;
			this.cbCmtJpeg = this.messages.UploadItem_cbCmtJpeg;
			this.cbCmtPng = this.messages.UploadItem_cbCmtPng;
			this.labelAddFile = this.messages.UploadItem_labelAddFile;
			this.labelEditFileProperties = this.messages.ResourcesEditor_labelEditFileProperties;
			this.editFilePropertiesIcon = dojo.moduleUrl("com.ibm.team.scm.web", "graphics/icons/properties_co.gif");
			this.removeIcon = dojo.moduleUrl("com.ibm.team.scm.web", "graphics/icons/remove.gif");
			this.params = params;
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				this.versionableDTO = params.versionableDTO;
			}
		},
		postCreate: function () {
			var that = this;
			if (this.params.hint === "TEXT_EDIT") {
				if (this.params.mode === "SINGLE_FILE_ADD") {
					// don't show general properties table when creating text file, there is nothing useful to show in there
					this._propertiesTables.style.display = "none";
					// show file name input specific to new text file case
					this._newTextFileDiv.style.display = "block";
					// redirect input for file name
					this._scmInputFileName = this._newTextFileName;				
				}
				// Disable so it isn't submitted
				this._scmInputFile.disabled = true;
				// Hide the section with the file picker
				this._addFile.style.display = "none";
				this.textEditorItemWidget = new com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.TextEditorItem(
				{},
					this._textEditor
				);
			} else {
				/**
				 * Disable (so it isn't submitted) the input text element
				 */
				this._scmInputContent.disabled = true;
			}
			this.connect(this._scmInputFileName, "onKeyUp", function (evt) {
				// Update save button - if user sets an empty name it is disabled
				that.getParent().onRefreshed();
			});			
			this.connect(this._scmInputFile, "onchange", function (evt) {
				evt.preventDefault();
				that._performSubmit();
			});
			this.connect(this._cancelUploadButton, "onclick", "_handleCancelClick");
			this.connect(this._removeButton, "onclick", function (evt) {
				var filler;
				evt.preventDefault();
				that.visibleBlocks(true, false, false);
				dojox.data.dom.removeChildren(that._itemSize);
				// IE - add a filler for border to show up
				filler = document.createElement("div");
				filler.innerHTML = "&nbsp;";
				that._itemSize.appendChild(filler);
				// Reset filename (in mod case, we use the previous dto's name anyway)
				if (this.params.mode === "SINGLE_FILE_ADD") {
					this._setFilename("");
					this.resetUserSettings();		
				}
			});
			this.connect(this._editFilePropertiesButton, "onclick", function (evt) {
				var checked, node;
				node = that._editFilePropertiesButton;
				if (dojo.hasClass(node, "disabled")) {
					evt.preventDefault();
					return;
				}
				checked = dojo.hasClass(node, "toggled");
				dojo.toggleClass(node, "toggled");
				that.visibleFilePropertiesBlocks(!checked);
				if (checked) {
					// update filename to display when closing file properties
					this._setFilename(this._scmInputFileName.attr("value"));
				}
			});
		},				
		startup: function () {
			this.initUserSelection();
			dojo.connect(this.getParent(), "onRefreshed", this, "onRefreshed");
			this.onRefreshed();
			if (this.params.hint === "TEXT_EDIT") {
				this.textEditorItemWidget.resetContent(this.versionableDTO);				
			}
		},
		showWidget: function () {
		},
		onRefreshed: function () {
			var lockStatus, parent;
			if (!this.domNode) {
				return;
			}
			try {
				parent = this.getParent();
			} catch (err) {
				// this happens when parent widget is destroyed (FF)
				return;
			}
			if (parent === null) {
				// this happens when parent widget is destroyed (IE)
				return;
			}
			// Lock status is only useful if updating an existing file
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				dojox.data.dom.removeChildren(this._lockStatusDiv);
				this._lockStatusDiv.appendChild(parent.getLockStatusHtmlNode());
			}
			
			// Enable the edit button if necessary, once the existing versionable has been fetched			
			// note edit button isn't enabled for the new text file case since everything is defaulted automatically to plain/text utf-8
			if ((this.params.mode === "SINGLE_FILE_UPLOAD" && this.versionableDTO)|| (this.params.mode === "SINGLE_FILE_ADD" && this.params.hint !== "TEXT_EDIT")) {
				dojo.removeClass(this._editFilePropertiesButton, "disabled");
			}
		},
		initUserSelection : function () {
			var i, modifiedBySpan, relativeDateSpan, date, contentType, value;
			if (this.params.mode === "SINGLE_FILE_UPLOAD") {
				this._scmFileName.appendChild(document.createTextNode(this.versionableDTO.versionable.name));
				value = ScmUtil.getVersionableDTOPropertyValue(this.versionableDTO, "fileTimestamp");
				date = new Date();
				date.setTime(value);
				modifiedBySpan = document.createElement("span");
				modifiedBySpan.className = "com-ibm-team-scm-uploadItem-modifiedBy-name";
				modifiedBySpan.appendChild(document.createTextNode(this.versionableDTO.creator.name));
				relativeDateSpan = document.createElement("span");
				relativeDateSpan.appendChild(document.createTextNode(jazz.util.date.formatDif(date, true, true)));
				relativeDateSpan.className = "com-ibm-team-scm-uploadItem-decorators-relativeDate";
				this._lastModifiedBy.appendChild(modifiedBySpan);
				this._lastModifiedBy.appendChild(relativeDateSpan);
				this._scmInputFileName.attr("value", this.versionableDTO.versionable.name);
				contentType = ScmUtil.getVersionableDTOPropertyValue(this.versionableDTO, "contentType");
				if (contentType) {
					this._scmSelectContentMimeType.attr('value', this._getContentMimeTypeLabel(contentType));
				}
				this._scmCheckBoxIsExecutable.attr('checked', this.versionableDTO.versionable.executable);
				this._scmSelectLineDelimiter.attr('value',
					this._getLineDelimiterLabel(this.versionableDTO.versionable.content.lineDelimiterSetting));
				this._scmSelectFileEncoding.attr('value', this.versionableDTO.versionable.content.characterEncoding);
			}
			if (this.params.mode === "SINGLE_FILE_ADD" && this.params.hint === "TEXT_EDIT") {
				this._scmSelectContentMimeType.attr('value',
								this._getContentMimeTypeLabel("text/plain"));
				this._scmSelectFileEncoding.attr('value', "UTF-8");
			}
		},
		_getLineDelimiterLabel: function (lineDelimiter) {
			if (lineDelimiter === 0) {
				return this.cbLdNoneBinary;
			}
			if (lineDelimiter === 1) {
				return this.cbLdLFUnix;
			}
			if (lineDelimiter === 2) {
				return this.cbLdCRMac;
			}
			if (lineDelimiter === 3) {
				return this.cbLdCRLFWindows;
			}
			if (lineDelimiter === 4) {
				return this.cbPlatform;
			}
			return this.cbLdNoneBinary;
		},
		_getContentMimeTypeLabel: function (contentType) {
			if (contentType === "application/octet-stream") {
				return this.cbLdNoneBinary;
			}
			if (contentType === "text/plain") {
				return this.cbCmtPlainText;
			}
			if (contentType === "text/html") {
				return this.cbCmtHtml;
			}
			if (contentType === "application/xml") {
				return this.cbCmtXml;
			}
			if (contentType === "image/jpeg") {
				return this.cbCmtJpeg;
			}
			if (contentType === "image/gif") {
				return this.cbCmtGif;
			}
			if (contentType === "image/png") {
				return this.cbCmtPng;
			}
			return contentType;
		},
		visibleFilePropertiesBlocks: function (show) {
			this._filePropertiesDiv.style.display = show ? "block" : "none";
			// and toggle the file name and decorators but not the progress cancel button
			this._scmFileName.style.display = show ? "none" : "block";
			this._lockStatusDiv.style.display = show ? "none" : "block";
		},
		
		visibleBlocks: function (showForm, showCancel, showRemove) {
			if (showForm) {
				this._scmDisabledInputFile.style.display = "none";
				this._scmInputFile.style.display = "block";
			} else {
				this._scmInputFile.style.display = "none";
				this._scmDisabledInputFile.style.display = "block";
			}
			this._uploadInProgressMessageArea.style.display = showCancel ? "block" : "none";
			dojo.toggleClass(this._removeButton, "disabled", !showRemove);
			
			// Notify parent we have reached a new state (e.g. parent needs to update save button state)
			this.getParent().onRefreshed();
		},
		/**
		 * With the TEXT_EDIT hint, upload the content edited by the user.
		 * Otherwise inspect if user changed the file properties after previous upload
		 * and re-upload as required.
		 * Returns right away if not necessary, notify via a Deferred when upload completed.
		 */
		checkAndRedoUpload : function () {
			var d, userSettings;
			if (this.params.hint !== "TEXT_EDIT") {
				userSettings = this.parseUserSettings();
				if (!this.uploadResult) {
					d = new dojo.Deferred();
					d.errback(new Error(this.messages.UploadItem_notUploaded));
					return d;
				}
				if (this.uploadResult.contentMimeType === userSettings.contentMimeType &&
				this.uploadResult.lineDelimiter === userSettings.lineDelimiter &&
				this.uploadResult.textFileEncoding === userSettings.textFileEncoding) {
					d = new dojo.Deferred();
					d.callback(true);
					return d;
				}
			}
			return this._performSubmit();				
		},
		/**
		 * Show the given name in the name control and in the input name control.
		 */
		_setFilename: function (filename) {
			this._scmFileName.innerHTML = filename;
			this._scmInputFileName.attr("value", filename);
		},
		_performSubmit: function () {
			var that, userSettings, filename, d, text;
			d = new dojo.Deferred();
			that = this;
			this.visibleBlocks(false, true, false);
			userSettings = this.parseUserSettings();				
			this._scmInputFileEncoding.value = userSettings.textFileEncoding;
			this._scmInputContentType.value = userSettings.contentMimeType;
			// Default to none/binary (0) if text file encoding is not set
			this._scmInputLineDelimiter.value = userSettings.textFileEncoding.length === 0 ? 0 : userSettings.lineDelimiter;
			// Handle the text edit case
			if (!this._scmInputContent.disabled) {
				text = this.textEditorItemWidget && this.textEditorItemWidget.getText();
				if (text) {
					this._scmInputContent.value = text;
				}
				filename = this._scmFileName.innerHTML;
			}
			else {
				// File picker case
				
				// Use file picker filename if this is an add case and name hasn't been manually set already
				if (this.params.mode === "SINGLE_FILE_ADD" && this._scmInputFileName.attr("value").length === 0) {
					// Note. IE returns a full path, extract the file name
					filename = ScmUtil.parseFileName(this._scmInputFile.value);
					this._setFilename(filename);
				}
				// otherwise, leave name as is
			}
			
			this.currentRequest = dojo.io.iframe.send({
				url : TeamServerClient._getServicePrefix() + "com.ibm.team.filesystem.service.internal.rest.IFilesystemContentService",
				handleAs: "text",
				form : this._uploadForm,
				error: function (response, ioArgs) {
					if (response.dojoType && response.dojoType === "cancel") {
						return; // cancelled request, ignore it
					}
					that._uploadFailed(response.message);
					d.errback(new Error(response.message));
				},
				handle: function (response, ioArgs) {
					var result;
					if (response instanceof Error) {
						if (response.dojoType && response.dojoType === "cancel") {
							return; // cancelled request, ignore it
						}
						that._uploadFailed(response.message);
						d.errback(new Error(response.message));
						return;
					}
					result = that.parseErrorResponse(response);
					if (result) {
						that._uploadFailed(result.error);
						d.errback(new Error(result.error));
						return;
					}
					that.uploadResult = that.parseUploadResponse(response);						
					if (!that.uploadResult) {
						that._uploadFailed(null);
						d.errback(new Error(that.messages.UploadItem_UploadFailed));
						return;
					}
					// Show server's reply
					that._itemSize.innerHTML = ScmUtil.getSizeDisplay(that.uploadResult.contentLength, true);
					if (that._scmInputFileEncoding.value === "") {
						// User has let the browser or server guess
						that.updateUserSettings(that.uploadResult);
					}
					that.visibleBlocks(false, false, true);
					d.callback(true);
				}
			});
			return d;
		},
		_handleCancelClick: function (e) {
			e.preventDefault();
			if (this.currentRequest) {
				this.currentRequest.cancel();
			}
			this._uploadForm.reset();
			// Reset filename (in mod case, we use the previous dto's name anyway)
			if (this.params.mode === "SINGLE_FILE_ADD") {
				this._setFilename("");
			}
			this.visibleBlocks(true, false, false);
		},
		_uploadFailed: function (serverErrorMessage) {
			this.currentRequest = null; // no need to cancel it anymore
			
			// Upload failed, warn user and allow for another attempt
			alert(serverErrorMessage ? serverErrorMessage : this.messages.UploadItem_UploadFailed);
			this._uploadForm.reset();
			this.visibleBlocks(true, false, false);
		},
		/**
		 * An object describing the key / values returned by the server
		 * and those set by the user. Null if upload isn't complete yet.
		 */
		getUploadValues: function () {
			var r, userSettings;
			r = this.uploadResult;
			if (!r) {
				return null;
			}
			userSettings = this.parseUserSettings();
			return {
				contentHash : r.contentHash,
				contentLength : r.contentLength,
				lineDelimiterCount : r.lineDelimiterCount,
				contentMimeType : r.contentMimeType,
				textFileEncoding : r.textFileEncoding,
				lineDelimiter : r.lineDelimiter,
				filename : userSettings.filename,
				executable : userSettings.executable
			};
		},
		parseErrorResponse: function (serverResponse) {
			var regexp, result;
			/**
			 * Parse the error returned by the server:
			 * File Upload Response:error:"value1",argument:"value2"
			 */
			regexp = /^File Upload Response:error:"([^"]*)",argument:"([^"]*)"$/;
			
			result = regexp.exec(serverResponse);
			if (result && result.length === 3) {
				return {
					error : result[1],
					argument : result[2]
				};
			}
			return null;
		},
		parseUploadResponse: function (serverResponse) {
			var regexp, result;
			/**
			 * Parse the answer returned by the server:
			 * File Upload Response:arg1:"value1",arg2:"value2",arg3:"value3"
			 */
			regexp = /^File Upload Response:contentHash:"([^"]*)",contentLength:"([^"]*)",lineDelimiterCount:"([^"]*)",contentType:"([^"]*)",encoding:"([^"]*)",lineDelimiter:"([^"]*)"$/;
			
			result = regexp.exec(serverResponse);
			if (result && result.length === 7) {
				return {
					contentHash : result[1],
					contentLength : result[2],
					lineDelimiterCount : parseInt(result[3], 10),
					contentMimeType : result[4],
					textFileEncoding : result[5],
					lineDelimiter : parseInt(result[6], 10)
				};
			}
			return null;
		},
		resetUserSettings: function () {
			this._scmSelectContentMimeType.attr("value", this.cbCmtAutoDetect);
			this._scmSelectLineDelimiter.attr("value", this.cbLdNoneBinary);
			this._scmSelectFileEncoding.attr("value", "");
		},	
		// Update file settings using server's response
		updateUserSettings: function (uploadResult) {
			this._scmSelectContentMimeType.attr("value", uploadResult.contentMimeType);
			if (uploadResult.lineDelimiter === 0) {
				this._scmSelectLineDelimiter.attr("value", this.cbLdNoneBinary);
			}
			if (uploadResult.lineDelimiter === 1) {
				this._scmSelectLineDelimiter.attr("value", this.cbLdLFUnix);
			}
			if (uploadResult.lineDelimiter === 2) {
				this._scmSelectLineDelimiter.attr("value", this.cbLdCRMac);
			}
			if (uploadResult.lineDelimiter === 3) {
				this._scmSelectLineDelimiter.attr("value", this.cbLdCRLFWindows);
			}
			if (uploadResult.lineDelimiter === 4) {
				this._scmSelectLineDelimiter.attr("value", this.cbPlatform);
			}
			this._scmSelectFileEncoding.attr("value", uploadResult.textFileEncoding);
		},
		parseUserSettings: function () {
			var result = {
				contentMimeType : "",
				textFileEncoding : "",
				lineDelimiter : 0,
				filename : "",
				executable : false
			};
			// Allow user to type their own
			result.contentMimeType = this._scmSelectContentMimeType.value;
			// Or recognize the predefined ones that have extra info		
			if (this._scmSelectContentMimeType.value === this.cbCmtAutoDetect) {
				result.contentMimeType = "";
			}
			if (this._scmSelectContentMimeType.value === this.cbLdNoneBinary) {
				result.contentMimeType = "application/octet-stream";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtPlainText) {
				result.contentMimeType = "text/plain";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtHtml) {
				result.contentMimeType = "text/html";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtXml) {
				result.contentMimeType = "application/xml";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtJpeg) {
				result.contentMimeType = "image/jpeg";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtGif) {
				result.contentMimeType = "image/gif";
			}
			if (this._scmSelectContentMimeType.value === this.cbCmtPng) {
				result.contentMimeType = "image/png";
			}
			if (this._scmSelectLineDelimiter.value === this.cbLdNoneBinary) {
				result.lineDelimiter = 0;
			}
			if (this._scmSelectLineDelimiter.value === this.cbLdLFUnix) {
				result.lineDelimiter = 1;
			}
			if (this._scmSelectLineDelimiter.value === this.cbLdCRMac) {
				result.lineDelimiter = 2;
			}
			if (this._scmSelectLineDelimiter.value === this.cbLdCRLFWindows) {
				result.lineDelimiter = 3;
			}
			if (this._scmSelectLineDelimiter.value === this.cbPlatform) {
				result.lineDelimiter = 4;
			}
			result.textFileEncoding = this._scmSelectFileEncoding.value;
			result.filename = this._scmInputFileName.attr("value");
			result.executable = this._scmCheckBoxIsExecutable.checked;
			return result;
		},
		destroy: function() {
			if (this.textEditorItemWidget) {
				this.textEditorItemWidget.destroy();
				this.textEditorItemWidget = null;
			}
			this.inherited(arguments);
		},
		/**
		 * Returns true if the state of this widget is sufficient to attempt a save
	 	* e.g. user successfully uploaded content, locking is fine.
	 	*/
		isSave: function () {
			// widget isn't ready yet
			if (!this.domNode) {
				return false;
			}
			// Empty names not allowed
			if (this._scmInputFileName.attr("value").length === 0) {
				return false;
			}
			if (this.params.hint === "TEXT_EDIT") {
				return true;
			}
			// user hasn't completed upload yet (form or upload message is still visible)
			if (this._scmInputFile.style.display === "block" || this._uploadInProgressMessageArea.style.display === "block") {
				return false;
			}
			return true;
		}
	}
	);
}());