/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");

dojo.require("dojo.i18n");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel");
dojo.require("com.ibm.rational.cm.web.ui.view.common.Tree");
dojo.require("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "NavigatorMessages");

(function(){
	
/*
 * Opens Navigator chooser dialog which allows to select queries from Navigator
 * Only one instance of the chooser dialog is instantiated, and it is reused
 * argument is a JavaScript object in the form:
 * {
 * 	filter: { tyle: "LIST", recordType: ... },
 * 	onOk: function(arr){},
 *	canSelectFolder: true/false, (specify if folder is allowed as selection or not. default is false)
 * 	parentDialog: dlg
 * }
 */
var _chooserDlg;
com.ibm.rational.cm.openNavigatorChooser = function(/* Object */ obj){
	// call loginController#process so that it checks for login before dialog is being showed
	com.ibm.rational.cm.cqLoginController.process({
		callback: function(){
			var tree;
			if(!_chooserDlg){
				// create navigtor tree
//				dojo["require"]("com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel");
				var props = com.ibm.rational.cm.getNavigatorProperties();
//				dojo["require"]("com.ibm.rational.cm.web.ui.view.common.Tree");
				tree = new com.ibm.rational.cm.web.ui.view.common.Tree(props);
				
				// create dialog
//				dojo["require"]("com.ibm.rational.cm.web.ui.view.common.ButtonDialog");
				_chooserDlg = new com.ibm.rational.cm.web.ui.view.common.ButtonDialog({
					id: "CqQueryChooserDialog",
					title: dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "NavigatorMessages").SelectQuery
				});
				_chooserDlg.setContent(tree);
				_chooserDlg.resize({ w:500, h:350 }); // approx. same size as workItem chooser dialog
				_chooserDlg.startup();
			}else{
				tree = _chooserDlg.getContent();
			}
			
			// clear selection when showing the dialog every time
			// since OK button won't be enabled if same query was selected (onSelectionChanged doesn't fire)
			tree.clearSelection();
			
			// set multi select
			tree.multiSelect = (obj.multiSelect == true);
			
			// disable [OK] button when showing the dialog
			_chooserDlg.setOkEnabled(false);
			
			// by default, don't allow selecting folder
			var canSelectFolder = (obj.canSelectFolder) ? obj.canSelectFolder : false;
			// enable/disable [OK] button depending on type of node selected
			tree.onSelectionChanged = function(selected){
				var enabled = false;
				var sel;
				if(canSelectFolder){
					enabled = true;
				}else{
					for(var i = selected.length-1; i >= 0; i--){
						if(!selected[i].isExpandable){
							enabled = true;
							break;
						}
					};
				}
				_chooserDlg.setOkEnabled(enabled);
			}
			
			// set onOk action
			var onOk = obj.onOk;
			if(onOk && dojo.isFunction(onOk)){
				_chooserDlg.execute = function(){
					var items = dojo.map(tree.getSelection(), function(node){
						return node.item;
					});
					
					// filter-out folder if canSelectFolder=false
					if(!canSelectFolder){
						items = dojo.filter(items, function(item){
							return (item.type != "FOLDER");
						});
					}
					onOk(items, tree.model.store);
				}
			}else{
				console.error("onOk function is not defined");
			}
			
			// set filter
			tree.setFilter(obj.filter);
			
			// this must be before _chooserDlg.show
			_chooserDlg.setParentDialog(obj.parentDialog);
			
			_chooserDlg.show();
		}
	});
}

})();