/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanElement"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanningAttribute"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttributeIdentifier"); //$NON-NLS-1$

(function() {

var PlanningAttribute				= com.ibm.team.apt.client.PlanningAttribute;
var PlanningAttributeIdentifier		= com.ibm.team.apt.client.PlanningAttributeIdentifier;

dojo.declare("com.ibm.team.apt.client.PlanElement", null, { //$NON-NLS-1$
	
	__children: null,
	
	constructor: function() {
		this.__children= [];
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
		/**
	 * @param {PlanningAttribute} attribute
	 * @param {any} value
	 */
	setAttributeValue: function(attribute, value) {
		 this.__resolveAttribute(attribute).setValue(this, value);
	},

	/**
	 * @param {PlanningAttribute} attribute
	 * @type any
	 */
	getAttributeValue: function(attribute) {
		return this.__resolveAttribute(attribute).getValue(this);
	},

	equals: function(other) {
		throw new Error("equals not implemented"); //$NON-NLS-1$
	},

	/**
	 * @param {Function} visitor
	 */
	accept: function(visitor, thisObj) {
		if (visitor.call(thisObj, this)) {
			for (var i= 0; i < this.__children.length; i++) {
				this.__children[i].accept(visitor, thisObj);
			}
		}
	},

	/**
	 * @param {PlanElement} element
	 */
	addChild: function(element) {
		this.__children.push(element);
		this._childAdded(this, element);
	},

	/**
	 * @param {PlanDelta} delta
	 * @type Array
	 */
	getChildren: function() {
		return [].concat(this.__children);
	},

	/**
	 * @type Integer 
	 */
	getChildrenCount: function() {
		return this.__children.length;
	},

	/**
	 * @type Boolean
	 */
	hasChildren: function() {
		return this.__children.length > 0;
	},

	/**
	 * @param {PlanElement} element
	 */
	removeChild: function(element) {
		var index= dojo.indexOf(this.__children, element);
		if (index != -1) {
			this.__children.splice(index, 1);
			this._childRemoved(this, element);
			return true;
		}
		return false;
	},

	/**
	 * @type PlanElement
	 */
	getParent: function() {
		throw new Error("PlanElement.getParent() not implemented by subclass"); //$NON-NLS-1$
	},

	/**
	 * @param {PlanElement} newParent
	 */
	moveTo: function(newParent) {
		throw new Error("PlanElement.moveTo() not implemented by subclass"); //$NON-NLS-1$
	},

	/**
	 * @type Boolean
	 */
	isDirty: function() {
		throw new Error("PlanElement.isDirty() not implemented by subclass"); //$NON-NLS-1$
	},

	/**
	 * @type ProblemReport
	 */
	getCheckReport: function() {
		throw new Error("PlanElement.getCheckReport() not implemented by subclass"); //$NON-NLS-1$
	},

	/**
	 * @type PlanModel
	 */
	getPlanModel: function() {
		throw new Error("PlanElement.getPlanModel() not implemented by subclass"); //$NON-NLS-1$
	},

	// ---- spi ------------------------------------------------------------------------------------------------------------
	

	_childAdded: function(parent, child) {
	},
	
	_childRemoved: function(parent, child) {
	},

	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__resolveAttribute: function(attributeId) {
		return attributeId instanceof PlanningAttribute ? attributeId : this.getPlanModel().findAttribute(attributeId);
	},
	

	_clearChildren: function() {
		for (var i= 0; i < this.__children.length; i++) {
			this.__children[i]._clearChildren();;
		}
		this.__children.splice(0, this.__children.length);
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.client.PlanElement.CHILDREN= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.children"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanElement.CHILDREN_COUNT= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.childrenCount"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanElement.PLANCHECK_REPORT= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.verificationReport"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanElement.DIRTY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.dirty"); //$NON-NLS-1$

})();
