/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.schedule.WorktimeScheduler"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Timespan"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.internal.Instant"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;

var Timespan						= com.ibm.team.apt.client.internal.Timespan;
var Instant							= com.ibm.team.apt.client.internal.Instant;

dojo.declare("com.ibm.team.apt.client.internal.schedule.WorktimeScheduler", null, { //$NON-NLS-1$

	__iterator: null,

	constructor: function(timeIterator) {
		this.__iterator= timeIterator;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	schedule: function(earliestStart, duration, forward) {
		this.__iterator.reset(earliestStart, forward);

		var startTime, endTime, currentStart, currentEnd;

		var lowFn= forward ? this.__lowForward : this.__lowReverse;
		var highFn= forward ? this.__highForward : this.__highReverse;

		var rawDuration= Math.max(duration.getMilliseconds(), 1);
		
		startTime= currentStart= lowFn(this.__iterator.current());
		endTime= currentEnd= highFn(this.__iterator.current(), rawDuration);

		while (rawDuration > 0) {
			rawDuration-= Math.abs(currentEnd - currentStart);
			endTime= currentEnd;

			if (!this.__iterator.next()) {
				throw new Error("Not enough work time left"); //$NON-NLS-1$
			}
			
			currentStart= lowFn(this.__iterator.current());
			currentEnd= highFn(this.__iterator.current(), rawDuration);
		}

		var begin= new Instant(new Date(startTime));  
		var end= new Instant(new Date(endTime));  
		
		return forward ? new Timespan(begin, end) : new Timespan(end, begin);
	},
	
	workTime: function(timespan) {
		this.__iterator.reset(timespan.start, true);
		
		var workTime= 0;
		var maxEnd= timespan.end.getDate().getTime();
		var currentStart, currentEnd;
		
		do {
			currentStart= Math.min(this.__iterator.current().start.getDate().getTime(), maxEnd);
			currentEnd= Math.min(this.__iterator.current().end.getDate().getTime(), maxEnd);
			workTime+= Math.max(0, currentEnd - currentStart);
		} while (this.__iterator.next() && currentEnd != maxEnd);

		return workTime;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__lowForward: function(timespan) {
		return timespan.start.getDate().getTime();
	},
	
	__lowReverse: function(timespan) {
		return timespan.end.getDate().getTime();
	},

	__highForward: function(timespan, rawDuration) {
		return Math.min(timespan.end.getDate().getTime(), timespan.start.getDate().getTime() + rawDuration);
	},
	
	__highReverse: function(timespan, rawDuration) {
		return Math.max(timespan.start.getDate().getTime(), timespan.end.getDate().getTime() - rawDuration);
	},
	
	__sentinel: null // terminates this class definition
});

})();
