/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.model._EntryState"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.ui.model._ViewerUpdater"); //$NON-NLS-1$

( function() {

var getAddEntryRunnable= com.ibm.team.apt.ui.model._ViewerUpdater.getAddEntryRunnable;
var getRefreshEntryRunnable= com.ibm.team.apt.ui.model._ViewerUpdater.getRefreshEntryRunnable;
var getRemoveEntryRunnable= com.ibm.team.apt.ui.model._ViewerUpdater.getRemoveEntryRunnable;
var getResortEntryRunnable= com.ibm.team.apt.ui.model._ViewerUpdater.getResortEntryRunnable;
var getUpdateEntryRunnable= com.ibm.team.apt.ui.model._ViewerUpdater.getUpdateEntryRunnable;

// ---- state helpers --------------------------------------------------------------------------------------------------

var _true= function() { return true; };
var _false= function() { return false; };
var _identity= function() { return this; };
var _transition= function(target) { return function() {
	var args= [this];
	for (var i= 0; i < arguments.length; i++) {
		args.push(arguments[i]);
	}
	var result= {};
	target.apply(result, args);
	return result;
}; };
var _rootTransition= function(transition, ifVisible) {
	var checkVisibility= arguments.length > 1; 
	return function() { 
		return (!checkVisibility || (ifVisible && this.isVisible())) ? this.root[transition](arguments) : this; 
	}; 
};
var _target= function(target) { return function() { return target; }; };

// ---- transitions ----------------------------------------------------------------------------------------------------

var DefaultStateTransition= {
	show :_rootTransition("show", false), //$NON-NLS-1$
	hide :_rootTransition("hide", true), //$NON-NLS-1$
	update :_rootTransition("update", true), //$NON-NLS-1$
	resort :_rootTransition("resort", true), //$NON-NLS-1$
	refresh :_rootTransition("refresh", true), //$NON-NLS-1$
	remove :_rootTransition("remove"), //$NON-NLS-1$

	hasUpdates :_true
};

var ShowTransition= function(root) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _true;

	this.update= _identity;
	this.resort= _identity;
	this.refresh= _identity;

	this.isAdd= _true;
	this.includesChildUpdates= _true;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		if (entry.isRootEntry()) {
			// the root entry has changed -> we need to refresh the viewer completely
			viewerUpdates.push(getRefreshEntryRunnable(viewer, entry));
		} else {
			viewerUpdates.push(getAddEntryRunnable(viewer, entry));
		}
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(ShownState);
};

var HideTransition= function(root) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _false;

	this.isAdd= _false;
	this.includesChildUpdates= _true;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		viewerUpdates.push(getRemoveEntryRunnable(viewer, entry, true));
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(HiddenState);
};

var UpdateTransition= function(root, updateProperties) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _true;

	if (updateProperties != null) {
		this.update= function(properties) {
			// intercept to merge
			var propertyList= null;
			if (properties != null) {
				var i;
				var propertySet= {};
				for (i= 0; i < updateProperties.length; i++) {
					propertySet[updateProperties[i]]= true;
				}

				for (i= 0; i < properties.length; i++) {
					propertySet[properties[i]]= true;
				}
				
				propertyList= [];
				for ( var property in propertySet)
					propertyList.push(property);
			}

			return DefaultStateTransition.update.call(this, propertyList);
		};
	}

	this.resort= _transition(RefreshTransition, root);

	this.isAdd= _false;
	this.includesChildUpdates= _false;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		viewerUpdates.push(getUpdateEntryRunnable(viewer, entry, updateProperties));
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(ShownState);
};

var ResortTransition= function(root) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _true;

	this.update= _transition(RefreshTransition, root);

	this.isAdd= _false;
	this.includesChildUpdates= _false;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		viewerUpdates.push(getResortEntryRunnable(viewer, entry));
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(ShownState);
};

var RefreshTransition= function(root) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _true;

	this.update= _identity;
	this.resort= _identity;

	this.isAdd= _false;
	this.includesChildUpdates= _true;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		viewerUpdates.push(getRefreshEntryRunnable(viewer, entry));
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(ShownState);
};

var RemoveTransition= function(root) {
	dojo.mixin(this, DefaultStateTransition);
	this.root= root;
	this.isVisible= _false;

	this.isAdd= _false;
	this.includesChildUpdates= _true;

	this.collectUpdates= function(viewer, entry, viewerUpdates) {
		viewerUpdates.push(getRemoveEntryRunnable(viewer, entry, false));
		return undefined; // get rid of stupid JSDT error
	};
	this.getUpdatedState= _target(HiddenState);
};

// ---- states ---------------------------------------------------------------------------------------------------------

var ShownState= {
	isVisible :_true,

	show :_identity,
	hide :_transition(HideTransition),
	update :_transition(UpdateTransition),
	resort :_transition(ResortTransition),
	refresh :_transition(RefreshTransition),
	remove :_transition(RemoveTransition),

	hasUpdates :_false,
	isAdd :_false,
	includesChildUpdates :_false,

	collectUpdates : function(viewer, entry, viewerUpdates) {},
	getUpdatedState :_identity
};

var HiddenState= dojo.mixin(dojo.mixin( {}, ShownState), {
	isVisible :_false,

	show :_transition(ShowTransition),
	hide :_identity,
	update :_identity,
	resort :_identity,
	refresh :_identity
});

com.ibm.team.apt.ui.model._EntryState.INITIAL= new ShowTransition(HiddenState);

})();
